from dataclasses import dataclass, field
from typing import Dict, Optional


@dataclass
class ScheduleConfig:
    epochs: int = 3
    ema: Optional[Dict] = field(default=None)
    optimizer: Dict = field(
        default_factory=lambda: {
            "name": "adamw",
            "lr": 6e-5,
            "betas": [0.9, 0.999],
            "weight_decay": 0.0005,
            "no_bias_decay": False,
            "no_norm_weight_decay": False,
            "overwrite": None,
        }
    )
    scheduler: Dict = field(
        default_factory=lambda: {
            "name": "cosine_no_sgdr",
            "warmup_epochs": 5,
            "warmup_bias_lr": 1e-5,
            "min_lr": 0.0,
        }
    )


@dataclass
class ClassificationScheduleConfig(ScheduleConfig):
    pass


@dataclass
class SegmentationScheduleConfig(ScheduleConfig):
    pass


@dataclass
class DetectionScheduleConfig(ScheduleConfig):
    pass



from .benchmarker import BenchmarkerMetadata
from .compressor import CompressorMetadata
from .converter import ConverterMetadata
from .trainer import TrainerMetadata

__all__ = ["TrainerMetadata", "CompressorMetadata", "ConverterMetadata", "BenchmarkerMetadata"]



from dataclasses import asdict, dataclass, field
from typing import List, Optional

from netspresso.enums.model import Framework
from netspresso.exceptions.compressor import NotFillInputLayersException


@dataclass
class InputLayer:
    name: str = "input"
    batch: int = 1
    channel: int = 3
    dimension: List[int] = field(default_factory=list)


@dataclass
class RequestCreateModel:
    object_name: str


@dataclass
class RequestUploadModel:
    url: str


@dataclass
class RequestValidateModel:
    input_layers: List[InputLayer]
    display_name: Optional[str] = ""
    framework: Framework = Framework.PYTORCH

    def __post_init__(self):
        new_input_layers = list(self.input_layers)
        if self.framework == Framework.PYTORCH and not new_input_layers:
            raise NotFillInputLayersException()
        self.input_layers = new_input_layers



from .base import InputLayer, ModelBase, ModelStatus
from .request_body import RequestCreateModel, RequestUploadModel, RequestValidateModel
from .response_body import (
    ResponseModelItem,
    ResponseModelItems,
    ResponseModelOptions,
    ResponseModelStatus,
    ResponseModelUrl,
)

__all__ = [
    ModelBase,
    InputLayer,
    ModelStatus,
    RequestCreateModel,
    RequestUploadModel,
    RequestValidateModel,
    ResponseModelUrl,
    ResponseModelItem,
    ResponseModelItems,
    ResponseModelStatus,
    ResponseModelOptions,
]



import json
import shutil
import sys
import zipfile
from pathlib import Path
from typing import Dict, List, Tuple, Union
from urllib import request

from loguru import logger

from netspresso.exceptions.common import NotSupportedFrameworkException, NotValidInputModelPath

FRAMEWORK_EXTENSION_MAP = {
    "tensorflow_keras": ".h5",
    "pytorch": ".pt",
    "onnx": ".pt",
    "tensorflow_lite": ".tflite",
    "drpai": ".zip",
    "openvino": ".zip",
    "tensorrt": ".trt",
}


class FileHandler:
    """Utility class for file-related operations."""

    @staticmethod
    def check_exists(folder_path: str) -> bool:
        """Check if the file or folder exists.

        Args:
            folder_path (str): The path to the folder.

        Returns:
            bool: True if the file or folder exists, False otherwise
        """
        return Path(folder_path).exists()

    @staticmethod
    def check_input_model_path(input_model_path: str):
        """Check if the input model path is a file.

        Args:
            input_model_path (str): The path to the input model file.

        Raises:
            FileNotFoundError: If the input model path is not a file.
        """

        if not Path(input_model_path).is_file():
            raise NotValidInputModelPath()

    @staticmethod
    def create_folder(
        folder_path: str,
        parents: bool = True,
        exist_ok: bool = True,
        is_folder_check: bool = True,
    ) -> None:
        """Create a folder.

        Args:
            folder_path (str): The path to the folder to be created.
            parents (bool, optional): If True, also create parent directories if they don't exist.
            exist_ok (bool, optional): If False, raise an error if the folder already exists.
            is_folder_check (bool, optional): If True, check if the folder already exists.

        Raises:
            SystemExit: If the folder already exists and `exist_ok` is False.
        """
        if is_folder_check and not FileHandler.check_exists(folder_path=folder_path):
            Path(folder_path).mkdir(parents=parents, exist_ok=exist_ok)
            logger.info(f"The folder has been created. Local Path: {Path(folder_path)}")
        elif is_folder_check:
            sys.exit(f"This folder already exists. Local Path: {Path(folder_path)}")

    @staticmethod
    def create_unique_folder(folder_path: str) -> Path:
        folder_path = Path(folder_path)
        if not folder_path.exists():
            folder_path.mkdir(parents=True)
            logger.info(f"The folder has been created. Local Path: {folder_path.as_posix()}")
        else:
            count = 1
            while True:
                new_folder_path = folder_path.with_name(f"{folder_path.name} ({count})")
                if not new_folder_path.exists():
                    new_folder_path.mkdir(parents=True)
                    folder_path = new_folder_path
                    logger.info(f"The folder has been created. Local Path: {folder_path.as_posix()}")
                    break
                count += 1

        return folder_path

    @staticmethod
    def create_file_path(folder_path: str, name: str, extension: str) -> Union[str, Path]:
        """Create a file path.

        Args:
            folder_path (str): The path to the folder where the file will be located.
            name (str): The name of the file.
            extension (str): The file extension.

        Returns:
            Union[str, Path]: The full path to the created file.
        """
        return Path(folder_path) / (name + extension)

    @staticmethod
    def download_file(url: str, save_path: Union[str, Path]) -> None:
        """Download a file from the given URL and save it to the specified path.

        Args:
            url (str): The URL of the file to be downloaded.
            save_path (Union[str, Path]): The path where the downloaded file will be saved.
        """
        request.urlretrieve(url, save_path)

    @staticmethod
    def get_extension_by_framework(framework: str) -> str:
        """Get the file extension based on the given framework.

        Args:
            framework (str): The framework name.

        Raises:
            KeyError: If the framework is not found in the extension map.

        Returns:
            str: The file extension corresponding to the framework.
        """
        extension = FRAMEWORK_EXTENSION_MAP.get(framework)
        if extension is None:
            available_frameworks = list(FRAMEWORK_EXTENSION_MAP.keys())
            raise NotSupportedFrameworkException(available_frameworks, framework)
        return extension

    @staticmethod
    def get_path_and_extension(folder_path: str, framework: str) -> Tuple[Path, str]:
        """Prepare the model path by creating folders and generating a default model path.

        Args:
            folder_path (str): The base folder path.
            framework (str): The framework name.
            is_folder_check (bool, optional): If True, check if the folder exists before creating.

        Returns:
            Tuple[Path, str]: A tuple containing the default model path (Path) and the file extension (str).
        """
        default_model_path = (Path(folder_path) / f"{Path(folder_path).name}.ext").resolve()
        extension = FileHandler.get_extension_by_framework(framework=framework)

        return default_model_path, extension

    @staticmethod
    def get_default_model_path(folder_path: str) -> Path:
        """Generates the default model file path based on the provided folder path.

        This method constructs a file path for the default model file located in the
        specified folder. The default model file is named using the name of the base
        folder and has a `.ext` extension. The resulting path is resolved to its absolute
        form.

        Args:
            folder_path (str): The base folder path where the model file is located.

        Returns:
            Path: The default model file path as a `Path` object. This path is formed by
                combining the base folder path with the default file name and extension,
                and is resolved to an absolute path.

        Example:
            >>> FileHandler.get_default_model_path('/models/my_model')
            PosixPath('/models/my_model/my_model.ext')

        Note:
            - Ensure that the base folder path provided is valid and accessible.
            - The extension `.ext` should be replaced with the actual file extension needed
            for the model file.
        """
        default_model_path = (Path(folder_path) / f"{Path(folder_path).name}.ext").resolve()

        return default_model_path

    @staticmethod
    def get_extension(framework: str) -> str:
        """Retrieves the file extension associated with the given framework.

        This method looks up the file extension for a specific framework using the
        `FRAMEWORK_EXTENSION_MAP`. If the framework is not found in the map, it raises
        a `KeyError` with a message listing all supported frameworks.

        Args:
            framework (str): The name of the framework for which the file extension is requested.

        Raises:
            KeyError: If the provided framework is not found in the `FRAMEWORK_EXTENSION_MAP`.
                    The error message includes the list of supported frameworks and the
                    entered framework.

        Returns:
            str: The file extension corresponding to the specified framework. This is a string
                representing the file extension (e.g., '.h5', '.pt', etc.).

        Example:
            >>> FileHandler.get_extension('pytorch')
            '.pt'

            >>> FileHandler.get_extension('unknown_framework')
            KeyError: "The framework supports ['tensorflow_keras', 'pytorch', 'onnx', 'tensorflow_lite', 'drpai', 'openvino', 'tensorrt']. The entered framework is unknown_framework."
        """
        extension = FRAMEWORK_EXTENSION_MAP.get(framework)
        if extension is None:
            available_frameworks = list(FRAMEWORK_EXTENSION_MAP.keys())
            raise NotSupportedFrameworkException(available_frameworks, framework)
        return extension

    @staticmethod
    def load_json(file_path: str):
        """Load JSON data from a file.

        Args:
            file_path (str): Path to the JSON file.

        Returns:
            dict: Loaded JSON data.
        """

        with open(file_path, "r") as json_data:
            data = json.load(json_data)
        return data

    @staticmethod
    def save_json(data: Union[Dict, List[Dict]], file_path: str) -> None:
        """Save data to a JSON file.

        Args:
            data (Union[Dict, List[Dict]]): The data to be saved. This can be a dictionary or a list of dictionaries.
            file_path (str): The file path where the JSON file will be saved.

        Returns:
            None: This function does not return any value.

        Notes:
            The data is written to a JSON file with indentation for readability. The file is saved in the specified directory with the given name.
        """

        with open(file_path, "w") as json_file:
            json.dump(data, json_file, indent=4)
        logger.info(f"JSON file saved at {file_path}")

    @staticmethod
    def move_and_cleanup_folders(source_folder: str, destination_folder: str):
        """Move files from the source folder to the destination folder and remove the source folder.

        Args:
            source_folder (str): The path to the source folder.
            destination_folder (str): The path to the destination folder.
        """

        source_folder = Path(source_folder)
        destination_folder = Path(destination_folder)

        for file_path in source_folder.iterdir():
            destination_path = destination_folder / file_path.name
            shutil.move(file_path, destination_path)

        source_folder.rmdir()

    @staticmethod
    def remove_folder(folder_path: str) -> None:
        """Remove a folder and its contents.

        Args:
            folder_path (str): Path to the folder.
        """

        folder_path = Path(folder_path)
        shutil.rmtree(folder_path, ignore_errors=True)

    @staticmethod
    def remove_file(file_path: str) -> None:
        """
        Remove a file.

        Args:
            file_path (str): Path to the file.
        """
        file_path = Path(file_path)

        if file_path.exists() and file_path.is_file():
            file_path.unlink()
            logger.info(f"File '{file_path}' has been removed.")
        else:
            logger.info(f"File '{file_path}' not found or is not a file.")

    @staticmethod
    def read_file_bytes(file_path: str) -> bytes:
        """Read the contents of a file and return them as bytes.

        Args:
            file_path (str): The path to the file to be read.

        Returns:
            bytes: The contents of the file as a byte object.
        """
        with open(file_path, "rb") as f:
            file_byte = f.read()
        return file_byte

    @staticmethod
    def unzip(zip_file_path: str, target_path: str) -> None:
        """Unzip a ZIP file and extract its contents to a specified directory.

        Args:
            zip_path (str): The path to the ZIP file that needs to be unzipped.
            target_path (str): The directory where the contents of the ZIP file will be extracted.

        Returns:
            None
        """
        with zipfile.ZipFile(zip_file_path, "r") as zip_ref:
            zip_ref.extractall(target_path)

    @staticmethod
    def rename_file(old_file_path: str, new_file_path: str):
        """Rename a file if it exists.

        Args:
            old_file_path (str): The original file path.
            new_file_path (str): The new file path.

        Returns:
            None
        """
        old_file_path = Path(old_file_path)
        new_file_path = Path(new_file_path)

        if old_file_path.exists():
            old_file_path.rename(new_file_path)
            logger.info(f"File '{old_file_path.name}' has been successfully renamed to '{new_file_path.name}'.")
        else:
            logger.info(f"File '{old_file_path.name}' not found.")



from pathlib import Path
from typing import List, Optional, Union

import qai_hub as hub
from loguru import logger
from qai_hub import JobStatus
from qai_hub.client import CompileJob, Dataset, Device, InputSpecs
from qai_hub.public_rest_api import DatasetEntries

from netspresso.analytics import netspresso_analytics
from netspresso.enums import Status
from netspresso.metadata.converter import ConverterMetadata
from netspresso.np_qai.base import NPQAIBase
from netspresso.np_qai.options import CompileOptions
from netspresso.np_qai.options.common import normalize_device_name
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class NPQAIConverter(NPQAIBase):
    def convert_image_dict_to_list(self, image_dict):
        result = []
        for key, value in image_dict.items():
            batch, channel, *dimension = value
            result.append({"name": key, "batch": batch, "channel": channel, "dimension": dimension})
        return result

    def get_convert_task_status(self, convert_task_id: str) -> JobStatus:
        """
        Get the status of a convert task.

        Args:
            convert_task_id: The ID of the convert task to get the status of.

        Returns:
            JobStatus: The status of the convert task.

        Note:
            For details, see `JobStatus in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.JobStatus.html>`_.
        """
        job: CompileJob = hub.get_job(convert_task_id)
        status = job.get_status()

        return status

    def update_convert_task(self, metadata: ConverterMetadata) -> ConverterMetadata:
        """
        Update the convert task.

        Args:
            metadata: The metadata of the convert task.

        Returns:
            ConverterMetadata: The updated metadata of the convert task.
        """
        job: CompileJob = hub.get_job(metadata.convert_task_info.convert_task_uuid)
        status = job.wait()

        if status.success:
            logger.info(f"{status.symbol} {status.state.name}")
            self.download_model(job=job, filename=metadata.converted_model_path)
            target_model = job.get_target_model()
            metadata.convert_task_info.output_model_uuid = target_model.model_id
            metadata.convert_task_info.data_type = job.target_shapes["image"][1]
            metadata.available_options = job.compatible_devices
            metadata.status = Status.COMPLETED
        elif status.failure:
            logger.info(f"{status.symbol} {status.state}: {status.message}")
            metadata.status = Status.ERROR
            metadata.update_message(exception_detail=status.message)

        MetadataHandler.save_metadata(data=metadata, folder_path=Path(metadata.converted_model_path).parent.as_posix())

        return metadata

    def convert_model(
        self,
        input_model_path: Union[str, Path],
        output_dir: str,
        target_device_name: Union[Device, List[Device]],
        input_shapes: Optional[InputSpecs] = None,
        options: Union[CompileOptions, str] = CompileOptions(),
        job_name: Optional[str] = None,
        single_compile: bool = True,
        calibration_data: Union[Dataset, DatasetEntries, str, None] = None,
        retry: bool = True,
    ) -> Union[ConverterMetadata, List[ConverterMetadata]]:
        """
        Convert a model in the QAI hub.

        Args:
            input_model_path: The path to the input model.
            output_dir: The directory to save the converted model.
            target_device_name: The device to compile the model for.
            input_shapes: The input shapes of the model.
            options: The options to use for the conversion.
            job_name: The name of the job.
            single_compile: Whether to compile the model in a single step.
            calibration_data: The calibration data to use for the conversion.
            retry: Whether to retry the conversion if it fails.

        Returns:
            Union[ConverterMetadata, List[ConverterMetadata]]: Returns a converter metadata object if successful.

        Note:
            For details, see `submit_compile_job in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.submit_compile_job.html>`_.
        """
        netspresso_analytics.send_event(
            event_name="convert_model",
            event_params={
                "target_device_name": normalize_device_name(target_device_name),
                "target_runtime": options.target_runtime.name,
                "quantize_full_type": options.quantize_full_type,
                "quantize_weight_type": options.quantize_weight_type,
                "compute_unit": options.normalize_compute_units(),
            },
        )

        output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
        default_model_path = (Path(output_dir) / f"{Path(output_dir).name}.ext").resolve()
        metadata = ConverterMetadata()
        metadata.input_model_path = Path(input_model_path).resolve().as_posix()
        extension = self.get_source_extension(model_path=input_model_path)
        metadata.model_info.framework = self.get_framework(extension=extension)

        MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        try:
            target_extension = self.get_target_extension(runtime=options.target_runtime)
            converted_model_path = default_model_path.with_suffix(target_extension).as_posix()

            cli_string = options.to_cli_string() if isinstance(options, CompileOptions) else options

            job = hub.submit_compile_job(
                model=input_model_path,
                device=target_device_name,
                name=job_name,
                input_specs=input_shapes,
                options=cli_string,
                single_compile=single_compile,
                calibration_data=calibration_data,
                retry=retry,
            )

            framework = self.get_framework_by_runtime(options.target_runtime)
            display_framework = self.get_display_framework(framework)

            metadata.model_info.input_shapes = self.convert_image_dict_to_list(input_shapes)
            metadata.model_info.data_type = job.shapes["image"][1]
            metadata.convert_task_info.convert_task_uuid = job.job_id
            metadata.converted_model_path = converted_model_path
            metadata.convert_task_info.input_model_uuid = job.model.model_id
            metadata.convert_task_info.device_name = target_device_name.name
            metadata.convert_task_info.display_device_name = target_device_name.name
            metadata.convert_task_info.framework = framework
            metadata.convert_task_info.display_framework = display_framework

            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        except KeyboardInterrupt:
            metadata.status = Status.STOPPED
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def download_model(self, job: CompileJob, filename: str):
        """
        Download a model from the QAI Hub.

        Args:
            job: The job to download the model from.
            filename: The filename to save the model to.

        Note:
            For details, see `download_target_model in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.CompileJob.html#qai_hub.CompileJob.download_target_model>`_.
        """
        job.download_target_model(filename=filename)



import time

from loguru import logger

from netspresso.clients.tao import tao_client
from netspresso.enums.tao.action import ConvertAction


class Dataset:
    def __init__(self, id, token_handler) -> None:
        self.id = id
        self.token_handler = token_handler
        self.job_map = {}
        self.convert_job_cnt = 1

    def update_dataset(self, name: str = None, description: str = None):
        try:
            logger.info("Updating dataset...")
            data = {}
            if name:
                data["name"] = name
            if description:
                data["description"] = description
            response = tao_client.dataset.update_dataset(
                self.token_handler.user_id, self.id, data, self.token_handler.headers
            )

            return response

        except Exception as e:
            logger.error(f"Update dataset failed. Error: {e}")
            raise e

    def delete_dataset(self):
        try:
            logger.info("Deleting dataset...")
            response = tao_client.dataset.delete_dataset(
                self.token_handler.user_id, self.id, self.token_handler.headers
            )

            return response

        except Exception as e:
            logger.error(f"Delete dataset failed. Error: {e}")
            raise e

    def get_dataset_convert_specs(self, action: ConvertAction):
        try:
            logger.info("Getting dataset convert specs...")
            response = tao_client.dataset.get_specs_schema(
                self.token_handler.user_id, self.id, action, self.token_handler.headers
            )
            dataset_convert_specs = response["default"]

            return dataset_convert_specs

        except Exception as e:
            logger.error(f"Convert dataset failed. Error: {e}")
            raise e

    def convert_dataset(self, parent_job_id, action: ConvertAction, specs):
        try:
            logger.info("Converting dataset...")
            data = {"parent_job_id": parent_job_id, "action": action, "specs": specs}
            convert_job_id = tao_client.dataset.run_dataset_jobs(
                self.token_handler.user_id, self.id, data, self.token_handler.headers
            )
            job_key = f"convert_job_{self.convert_job_cnt}"
            self.job_map[job_key] = convert_job_id
            self.convert_job_cnt += 1

            return convert_job_id

        except Exception as e:
            logger.error(f"Convert dataset failed. Error: {e}")
            raise e

    def monitor_job_status(self, job_id, interval=15):
        try:
            logger.info("Monitoring dataset job stauts...")

            while True:
                response = tao_client.dataset.get_dataset_job(
                    self.token_handler.user_id, self.id, job_id, self.token_handler.headers
                )
                logger.info(response)
                if response.get("status") in ["Done", "Error", "Canceled"]:
                    break
                time.sleep(interval)

        except Exception as e:
            logger.error(f"Monitor dataset job staus failed. Error: {e}")
            raise e

        except KeyboardInterrupt:
            logger.info("End monitoring.")

    def get_dataset_jobs(self, skip=None, size=None, sort=None):
        try:
            logger.info("Getting dataset jobs...")
            response = tao_client.dataset.get_dataset_jobs(
                self.token_handler.user_id, self.id, self.token_handler.headers, skip, size, sort
            )

            return response

        except Exception as e:
            logger.error(f"Get dataset jobs failed. Error: {e}")
            raise e

    def get_dataset_job(self, job_id: str):
        try:
            logger.info("Getting dataset job...")
            response = tao_client.dataset.get_dataset_job(
                self.token_handler.user_id, self.id, job_id, self.token_handler.headers
            )

            return response

        except Exception as e:
            logger.error(f"Get dataset job. Error: {e}")
            raise e

    def delete_dataset_job(self, job_id: str):
        try:
            logger.info("Deleting dataset job...")
            response = tao_client.dataset.delete_dataset_job(
                self.token_handler.user_id, self.id, job_id, self.token_handler.headers
            )

            return response

        except Exception as e:
            logger.error(f"Delete dataset job failed. Error: {e}")
            raise e

    def cancel_dataset_job(self, job_id: str):
        try:
            logger.info("Canceling dataset job...")
            response = tao_client.dataset.cancel_dataset_job(
                self.token_handler.user_id, self.id, job_id, self.token_handler.headers
            )

            return response

        except Exception as e:
            logger.error(f"Cancel dataset job failed. Error: {e}")
            raise e



from pathlib import Path

from .netspresso import NPQAI, TAO, NetsPresso

__all__ = ["NetsPresso", "TAO", "NPQAI"]


version = (Path(__file__).parent / "VERSION").read_text().strip()

__version__ = version



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.trainer.models.base import ArchitectureConfig, CheckpointConfig, ModelConfig


@dataclass
class MobileViTArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "mobilevit",
            "params": {
                "patch_size": 2,
                "num_attention_heads": 4,
                "attention_dropout_prob": 0.1,
                "ffn_dropout_prob": 0.0,
                "output_expansion_ratio": 4,
                "use_fusion_layer": True,
            },
            "stage_params": [
                {
                    "block_type": "mv2",
                    "out_channels": 32,
                    "num_blocks": 1,
                    "stride": 1,
                    "ir_expansion_ratio": 4,  # [mv2_exp_mult] * 4
                },
                {
                    "block_type": "mv2",
                    "out_channels": 64,
                    "num_blocks": 3,
                    "stride": 2,
                    "ir_expansion_ratio": 4,  # [mv2_exp_mult] * 4
                },
                {
                    "block_type": "mobilevit",
                    "out_channels": 96,
                    "num_blocks": 2,
                    "stride": 2,
                    "attention_channels": 144,
                    "ffn_intermediate_channels": 288,
                    "dilate": False,
                    "ir_expansion_ratio": 4,  # [mv2_exp_mult] * 4
                },
                {
                    "block_type": "mobilevit",
                    "out_channels": 128,
                    "num_blocks": 4,
                    "stride": 2,
                    "attention_channels": 192,
                    "ffn_intermediate_channels": 384,
                    "dilate": False,
                    "ir_expansion_ratio": 4,  # [mv2_exp_mult] * 4
                },
                {
                    "block_type": "mobilevit",
                    "out_channels": 160,
                    "num_blocks": 3,
                    "stride": 2,
                    "attention_channels": 240,
                    "ffn_intermediate_channels": 480,
                    "dilate": False,
                    "ir_expansion_ratio": 4,  # [mv2_exp_mult] * 4
                },
            ],
        }
    )


@dataclass
class ClassificationMobileViTModelConfig(ModelConfig):
    task: str = "classification"
    name: str = "mobilevit_s"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MobileViTArchitectureConfig(
            head={
                "name": "fc",
                "params": {
                    "num_layers": 1,
                    "intermediate_channels": None,
                    "act_type": None,
                    "dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "cross_entropy", "label_smoothing": 0.1, "weight": None}]
    )



from dataclasses import dataclass

from netspresso.enums.base import StrEnum


@dataclass
class AbstractResponse:
    pass


@dataclass
class PagingResponse(AbstractResponse):
    total_count: int
    result_count: int


class MembershipType(StrEnum):
    BASIC = "BASIC"
    PRO = "PRO"
    PREMIUM = "PREMIUM"


class CreditType(StrEnum):
    FREE = "FREE"
    PAID = "PAID"



from dataclasses import dataclass, field
from pathlib import Path
from typing import Any, Dict, List, Literal, Optional, Union

from omegaconf import MISSING, MissingMandatoryValue

__all__ = [
    "DatasetConfig",
    "LocalClassificationDatasetConfig",
    "LocalSegmentationDatasetConfig",
    "LocalDetectionDatasetConfig",
    "HuggingFaceClassificationDatasetConfig",
    "HuggingFaceSegmentationDatasetConfig",
    "ExampleBeansDataset",
    "ExampleChessDataset",
    "ExampleCocoyoloDataset",
    "ExampleSidewalkDataset",
    "ExampleXrayDataset",
    "ExampleSidewalkDataset",
    "ExampleSkincancerDataset",
    "ExampleTrafficsignDataset",
    "ExampleVoc12Dataset",
    "ExampleVoc12CustomDataset",
    "ExampleWikiartDataset",
]


@dataclass
class DatasetConfig:
    name: str = MISSING
    task: str = MISSING
    format: str = MISSING  # Literal['huggingface', 'local']


@dataclass
class ImageLabelPathConfig:
    image: Optional[Union[Path, str]] = None
    label: Optional[Union[Path, str]] = None


@dataclass
class PathPatternConfig:
    image: Optional[str] = None
    label: Optional[str] = None


@dataclass
class PathConfig:
    root: Union[Path, str] = MISSING
    train: ImageLabelPathConfig = field(default_factory=lambda: ImageLabelPathConfig(image=MISSING))
    valid: ImageLabelPathConfig = field(default_factory=lambda: ImageLabelPathConfig())
    test: ImageLabelPathConfig = field(default_factory=lambda: ImageLabelPathConfig())
    pattern: PathPatternConfig = field(default_factory=lambda: PathPatternConfig())


@dataclass
class HuggingFaceConfig:
    custom_cache_dir: Optional[
        Union[Path, str]
    ] = None  # If None, it follows HF datasets default (.cache/huggingface/datasets)
    repo: str = MISSING
    subset: Optional[str] = None
    features: Dict[str, str] = field(default_factory=lambda: {"image": "image", "label": "labels"})


@dataclass
class LocalClassificationDatasetConfig(DatasetConfig):
    task: str = "classification"
    format: str = "local"
    path: PathConfig = field(default_factory=lambda: PathConfig())
    id_mapping: Optional[List[str]] = None


@dataclass
class LocalSegmentationDatasetConfig(DatasetConfig):
    task: str = "segmentation"
    format: str = "local"
    path: PathConfig = field(default_factory=lambda: PathConfig())
    label_image_mode: str = "L"
    id_mapping: Any = None
    pallete: Optional[List[List[int]]] = None


@dataclass
class LocalDetectionDatasetConfig(DatasetConfig):
    task: str = "detection"
    format: str = "local"
    path: PathConfig = field(default_factory=lambda: PathConfig())
    id_mapping: Any = None
    pallete: Optional[List[List[int]]] = None


@dataclass
class HuggingFaceClassificationDatasetConfig(DatasetConfig):
    task: str = "classification"
    format: str = "huggingface"
    metadata: HuggingFaceConfig = field(
        default_factory=lambda: HuggingFaceConfig(features={"image": "image", "label": "labels"})
    )
    id_mapping: Optional[List[str]] = None


@dataclass
class HuggingFaceSegmentationDatasetConfig(DatasetConfig):
    task: str = "segmentation"
    format: str = "huggingface"
    metadata: HuggingFaceConfig = field(
        default_factory=lambda: HuggingFaceConfig(features={"image": "pixel_values", "label": "label"})
    )
    label_image_mode: str = "L"
    id_mapping: Any = None
    pallete: Optional[List[List[int]]] = None


ExampleBeansDataset = HuggingFaceClassificationDatasetConfig(
    name="beans",
    metadata=HuggingFaceConfig(custom_cache_dir=None, repo="beans", features={"image": "image", "label": "labels"}),
)

ExampleChessDataset = LocalClassificationDatasetConfig(
    name="chess",
    path=PathConfig(
        root="/DATA/classification-example",
        train=ImageLabelPathConfig(image="train"),
        valid=ImageLabelPathConfig(image="val"),
    ),
)

ExampleXrayDataset = HuggingFaceClassificationDatasetConfig(
    name="chest_xray_classification",
    metadata=HuggingFaceConfig(
        custom_cache_dir=None,
        repo="keremberke/chest-xray-classification",
        subset="full",
        features={"image": "image", "label": "labels"},
    ),
)

ExampleCocoyoloDataset = LocalDetectionDatasetConfig(
    name="coco_for_yolo_model",
    path=PathConfig(
        root="/DATA/coco",
        train=ImageLabelPathConfig(image="images/train2017", label="labels/train2017"),
        valid=ImageLabelPathConfig(image="images/train2017", label="labels/train2017"),
        pattern=PathPatternConfig(image="([0-9]{12})\\.jpg", label="([0-9]{12})\\.txt"),
    ),
    id_mapping=[
        "person",
        "bicycle",
        "car",
        "motorcycle",
        "airplane",
        "bus",
        "train",
        "truck",
        "boat",
        "traffic light",
        "fire hydrant",
        "stop sign",
        "parking meter",
        "bench",
        "bird",
        "cat",
        "dog",
        "horse",
        "sheep",
        "cow",
        "elephant",
        "bear",
        "zebra",
        "giraffe",
        "backpack",
        "umbrella",
        "handbag",
        "tie",
        "suitcase",
        "frisbee",
        "skis",
        "snowboard",
        "sports ball",
        "kite",
        "baseball bat",
        "baseball glove",
        "skateboard",
        "surfboard",
        "tennis racket",
        "bottle",
        "wine glass",
        "cup",
        "fork",
        "knife",
        "spoon",
        "bowl",
        "banana",
        "apple",
        "sandwich",
        "orange",
        "broccoli",
        "carrot",
        "hot dog",
        "pizza",
        "donut",
        "cake",
        "chair",
        "couch",
        "potted plant",
        "bed",
        "dining table",
        "toilet",
        "tv",
        "laptop",
        "mouse",
        "remote",
        "keyboard",
        "cell phone",
        "microwave",
        "oven",
        "toaster",
        "sink",
        "refrigerator",
        "book",
        "clock",
        "vase",
        "scissors",
        "teddy bear",
        "hair drier",
        "toothbrush",
    ],
)

ExampleSidewalkDataset = HuggingFaceSegmentationDatasetConfig(
    name="sidewalk_semantic",
    metadata=HuggingFaceConfig(
        custom_cache_dir=None, repo="segments/sidewalk-semantic", features={"image": "pixel_values", "label": "label"}
    ),
    label_image_mode="L",
    id_mapping=[
        "unlabeled",
        "flat-road",
        "flat-sidewalk",
        "flat-crosswalk",
        "flat-cyclinglane",
        "flat-parkingdriveway",
        "flat-railtrack",
        "flat-curb",
        "human-person",
        "human-rider",
        "vehicle-car",
        "vehicle-truck",
        "vehicle-bus",
        "vehicle-tramtrain",
        "vehicle-motorcycle",
        "vehicle-bicycle",
        "vehicle-caravan",
        "vehicle-cartrailer",
        "construction-building",
        "construction-door",
        "construction-wall",
        "construction-fenceguardrail",
        "construction-bridge",
        "construction-tunnel",
        "construction-stairs",
        "object-pole",
        "object-trafficsign",
        "object-trafficlight",
        "nature-vegetation",
        "nature-terrain",
        "sky",
        "void-ground",
        "void-dynamic",
        "void-static",
        "void-unclear",
    ],
)

ExampleSkincancerDataset = HuggingFaceClassificationDatasetConfig(
    name="skin_cancer",
    metadata=HuggingFaceConfig(
        custom_cache_dir=None, repo="marmal88/skin_cancer", features={"image": "image", "label": "dx"}
    ),
)

ExampleTrafficsignDataset = LocalDetectionDatasetConfig(
    name="traffic_sign_yolo",
    path=PathConfig(
        root="../../data/traffic-sign",
        train=ImageLabelPathConfig(image="images/train", label="labels/train"),
        valid=ImageLabelPathConfig(image="images/val", label="labels/val"),
    ),
    id_mapping=["prohibitory", "danger", "mandatory", "other"],  # class names
)

ExampleVoc12Dataset = LocalSegmentationDatasetConfig(
    name="voc2012",
    path=PathConfig(
        root="/DATA/VOC12Dataset",
        train=ImageLabelPathConfig(image="image/train", label="mask/train"),
        valid=ImageLabelPathConfig(image="image/val", label="mask/val"),
    ),
    label_image_mode="RGB",
    id_mapping={
        "(0, 0, 0)": "background",
        "(128, 0, 0)": "aeroplane",
        "(0, 128, 0)": "bicycle",
        "(128, 128, 0)": "bird",
        "(0, 0, 128)": "boat",
        "(128, 0, 128)": "bottle",
        "(0, 128, 128)": "bus",
        "(128, 128, 128)": "car",
        "(64, 0, 0)": "cat",
        "(192, 0, 0)": "chair",
        "(64, 128, 0)": "cow",
        "(192, 128, 0)": "diningtable",
        "(64, 0, 128)": "dog",
        "(192, 0, 128)": "horse",
        "(64, 128, 128)": "motorbike",
        "(192, 128, 128)": "person",
        "(0, 64, 0)": "pottedplant",
        "(128, 64, 0)": "sheep",
        "(0, 192, 0)": "sofa",
        "(128, 192, 0)": "train",
        "(0, 64, 128)": "tvmonitor",
        "(128, 64, 128)": "void",
    },
)

ExampleVoc12CustomDataset = LocalSegmentationDatasetConfig(
    name="voc2012",
    path=PathConfig(
        root="../../data/VOC12Dataset",
        train=ImageLabelPathConfig(image="image/train", label="mask/train"),
        valid=ImageLabelPathConfig(image="image/val", label="mask/val"),
    ),
    label_image_mode="L",
    id_mapping=[
        "background",
        "aeroplane",
        "bicycle",
        "bird",
        "boat",
        "bottle",
        "bus",
        "car",
        "cat",
        "chair",
        "cow",
        "diningtable",
        "dog",
        "horse",
        "motorbike",
        "person",
        "pottedplant",
        "sheep",
        "sofa",
        "train",
        "tvmonitor",
    ],
    pallete=[
        [0, 0, 0],
        [128, 0, 0],
        [0, 128, 0],
        [128, 128, 0],
        [0, 0, 128],
        [128, 0, 128],
        [0, 128, 128],
        [128, 128, 128],
        [64, 0, 0],
        [192, 0, 0],
        [64, 128, 0],
        [192, 128, 0],
        [64, 0, 128],
        [192, 0, 128],
        [64, 128, 128],
        [192, 128, 128],
        [0, 64, 0],
        [128, 64, 0],
        [0, 192, 0],
        [128, 192, 0],
        [0, 64, 128],
    ],
)

ExampleWikiartDataset = HuggingFaceClassificationDatasetConfig(
    name="wikiart_artist",
    metadata=HuggingFaceConfig(
        custom_cache_dir=None, repo="huggan/wikiart", subset="full", features={"image": "image", "label": "artist"}
    ),
)



from typing import Literal

from netspresso.enums.base import StrEnum


class DeviceName(StrEnum):
    RASPBERRY_PI_5 = "RaspberryPi5"
    RASPBERRY_PI_4B = "RaspberryPi4B"
    RASPBERRY_PI_3B_PLUS = "RaspberryPi3BPlus"
    RASPBERRY_PI_3B = "RaspberryPi3B"
    RASPBERRY_PI_2B = "RaspberryPi2B"
    RASPBERRY_PI_ZERO_W = "RaspberryPi-ZeroW"
    RASPBERRY_PI_ZERO_2W = "RaspberryPi-Zero2W"
    RENESAS_RZ_V2L = "rzv2l_avnet"
    RENESAS_RZ_V2M = "rzv2m"
    RENESAS_RA8D1 = "Renesas-RA8D1"

    JETSON_NANO = "Jetson-Nano"
    JETSON_TX2 = "Jetson-Tx2"
    JETSON_XAVIER = "Jetson-Xavier"
    JETSON_NX = "Jetson-Nx"
    JETSON_AGX_ORIN = "Jetson-AGX-Orin"
    JETSON_ORIN_NANO = "Jetson-Orin-Nano"
    AWS_T4 = "AWS-T4"
    INTEL_XEON_W_2233 = "Intel-Xeon"
    ALIF_ENSEMBLE_E7_DEVKIT_GEN2 = "Ensemble-E7-DevKit-Gen2"

    ARM_ETHOS_U_SERIES = "Arm Virtual Hardware Ethos-U Series"
    NXP_iMX93 = "nxp_imx93_ethos_u65"
    ARDUINO_NICLA_VISION = "arduino_nicla_vision"

    @classmethod
    def create_literal(cls):
        return Literal[
            "RaspberryPi5",
            "RaspberryPi4B",
            "RaspberryPi3BPlus",
            "RaspberryPi3B",
            "RaspberryPi2B",
            "RaspberryPi-ZeroW",
            "RaspberryPi-Zero2W",
            "rzv2l_avnet",
            "rzv2m",
            "Renesas-RA8D1",
            "Jetson-Nano",
            "Jetson-Tx2",
            "Jetson-Xavier",
            "Jetson-Nx",
            "Jetson-AGX-Orin",
            "Jetson-Orin-Nano",
            "AWS-T4",
            "Intel-Xeon",
            "Ensemble-E7-DevKit-Gen2",
            "Arm Virtual Hardware Ethos-U Series",
            "nxp_imx93_ethos_u65",
            "arduino_nicla_vision",
        ]


class SoftwareVersion(StrEnum):
    JETPACK_4_4_1 = "4.4.1-b50"
    JETPACK_4_6 = "4.6-b199"
    JETPACK_5_0_1 = "5.0.1-b118"
    JETPACK_5_0_2 = "5.0.2-b231"
    JETPACK_6_1 = "6.1+b123"

    @classmethod
    def create_literal(cls):
        return Literal["4.4.1-b50", "4.6-b199", "5.0.1-b118", "5.0.2-b231", "6.1+b123"]


class DisplaySoftwareVersion(StrEnum):
    JETPACK_4_4_1 = "Jetpack 4.4.1"
    JETPACK_4_6 = "Jetpack 4.6"
    JETPACK_5_0_1 = "Jetpack 5.0.1"
    JETPACK_5_0_2 = "Jetpack 5.0.2"
    JETPACK_6_1 = "Jetpack 6.1"


class HardwareType(StrEnum):
    HELIUM = "helium"

    @classmethod
    def create_literal(cls):
        return Literal["helium"]


class TaskStatus(StrEnum):
    IN_QUEUE = "IN_QUEUE"
    IN_PROGRESS = "IN_PROGRESS"
    FINISHED = "FINISHED"
    ERROR = "ERROR"
    TIMEOUT = "TIMEOUT"
    USER_CANCEL = "USER_CANCEL"

    @classmethod
    def create_literal(cls):
        return Literal["IN_QUEUE", "IN_PROGRESS", "FINISHED", "ERROR", "USER_CANCEL", "TIMEOUT"]


device_name_literal = DeviceName.create_literal()



from pathlib import Path

from loguru import logger

from netspresso.clients.tao import TAOTokenHandler, tao_client
from netspresso.enums.tao.experiment import CheckpointChooseMethod, EncryptionKey, NetworkArch
from netspresso.tao.dataset import Dataset
from netspresso.tao.experiment import Experiment
from netspresso.tao.utils.file import split_tar_file


class TAOTrainer:
    def __init__(self, token_handler: TAOTokenHandler) -> None:
        self.token_handler = token_handler

    def upload_dataset(
        self, name: str, dataset_type: str, dataset_format: str, dataset_path: str, split_name: str = "train"
    ):
        try:
            logger.info("Creating dataset...")
            data = {"name": name, "type": dataset_type, "format": dataset_format}
            response = tao_client.dataset.create_dataset(self.token_handler.user_id, data, self.token_handler.headers)
            dataset_id = response["id"]

            dataset_path = Path(dataset_path)
            output_dir = dataset_path.parent / "split" / split_name
            split_tar_file(dataset_path, str(output_dir))

            for idx, tar_dataset_path in enumerate(output_dir.iterdir()):
                logger.info(f"Uploading {idx+1}/{len(list(output_dir.iterdir()))} tar split")
                upload_dataset_response = tao_client.dataset.upload_dataset(
                    self.token_handler.user_id, dataset_id, str(tar_dataset_path), self.token_handler.headers
                )
                logger.info(upload_dataset_response["message"])

            dataset = Dataset(id=response["id"], token_handler=self.token_handler)

            return dataset

        except Exception as e:
            logger.error(f"Upload dataset failed. Error: {e}")
            raise e

    def get_datasets(self, skip=None, size=None, sort=None, name=None, format=None, type=None):
        try:
            logger.info("Getting datasets...")
            response = tao_client.dataset.get_datasets(
                self.token_handler.user_id, self.token_handler.headers, skip, size, sort, name, format, type
            )

            return response

        except Exception as e:
            logger.error(f"Get datasets failed. Error: {e}")
            raise e

    def get_dataset(self, dataset_id: str):
        try:
            logger.info("Getting dataset...")
            response = tao_client.dataset.get_dataset(
                self.token_handler.user_id, dataset_id, self.token_handler.headers
            )

            dataset = Dataset(id=response["id"], token_handler=self.token_handler)

            return dataset

        except Exception as e:
            logger.error(f"Get dataset failed. Error: {e}")
            raise e

    def create_experiment(
        self,
        name: str,
        network_arch: NetworkArch,
        encryption_key: EncryptionKey,
        checkpoint_choose_method: CheckpointChooseMethod = CheckpointChooseMethod.BEST_MODEL,
    ):
        try:
            logger.info("Creating experiment...")
            data = {
                "name": name,
                "network_arch": network_arch,
                "encryption_key": encryption_key,
                "checkpoint_choose_method": checkpoint_choose_method,
            }
            response = tao_client.experiment.create_experiments(
                self.token_handler.user_id, data, self.token_handler.headers
            )
            experiment = Experiment(
                name=name, id=response["id"], network_arch=network_arch, token_handler=self.token_handler
            )
            logger.info(f"Created experiment. experiment id: {experiment.id}")

            return experiment

        except Exception as e:
            logger.error(f"Create experiment failed. Error: {e}")
            raise e

    def get_experiments(
        self, skip=None, size=None, sort=None, name=None, type=None, network_arch=None, read_only=None, user_only=None
    ):
        try:
            logger.info("Getting experiments...")
            experiments = tao_client.experiment.get_experiments(
                self.token_handler.user_id,
                self.token_handler.headers,
                skip,
                size,
                sort,
                name,
                type,
                network_arch,
                read_only,
                user_only,
            )

            return experiments

        except Exception as e:
            logger.error(f"Get train schema failed. Error: {e}")
            raise e

    def get_experiment(self, experiment_id):
        try:
            logger.info("Getting experiment...")
            response = tao_client.experiment.get_experiment(
                self.token_handler.user_id, experiment_id, self.token_handler.headers
            )
            experiment = Experiment(
                name=response["name"],
                id=response["id"],
                network_arch=response["network_arch"],
                token_handler=self.token_handler,
            )

            return experiment

        except Exception as e:
            logger.error(f"Get experiment failed. Error: {e}")
            raise e



import json
from dataclasses import asdict, dataclass, field
from typing import Dict, List, Optional, Union

from netspresso.clients.launcher.v2.schemas import InputLayer
from netspresso.enums import QuantizationMode, QuantizationPrecision, SimilarityMetric


@dataclass
class QuantizationOptions:
    use_cuda: bool = False
    suggestion_only: bool = False
    metric: SimilarityMetric = SimilarityMetric.SNR  # Quantization quality metrics
    threshold: Union[float, int] = 0  # Quantization quality threshold
    weight_precision: QuantizationPrecision = QuantizationPrecision.INT8  # Weight precision
    activation_precision: QuantizationPrecision = QuantizationPrecision.INT8  # Activation precision

    def __post_init__(self):
        # weight_precision validation
        if self.weight_precision not in [QuantizationPrecision.INT8, QuantizationPrecision.FLOAT32]:
            if self.weight_precision == QuantizationPrecision.FLOAT16:
                raise ValueError("weight_precision FLOAT16 is only available for custom precision quantization.")
            raise ValueError(f"weight_precision must be either INT8 or FLOAT32, got {self.weight_precision}")

        # activation_precision validation
        if self.activation_precision not in [QuantizationPrecision.INT8, QuantizationPrecision.FLOAT32]:
            if self.activation_precision == QuantizationPrecision.FLOAT16:
                raise ValueError("activation_precision FLOAT16 is only available for custom precision quantization.")
            raise ValueError(f"activation_precision must be either INT8 or FLOAT32, got {self.activation_precision}")


@dataclass
class PlainQuantizationOption(QuantizationOptions):
    pass


@dataclass
class RecommendationOption(QuantizationOptions):
    suggestion_only: bool = True


@dataclass
class CustomQuantizeOption:
    use_cuda: bool = False
    metric: SimilarityMetric = SimilarityMetric.SNR  # Quantization quality metrics
    custom_precision: Dict = field(default_factory=dict)  # Custom precision by layer or operator
    weight_precision: QuantizationPrecision = QuantizationPrecision.INT8  # Weight precision
    activation_precision: QuantizationPrecision = QuantizationPrecision.INT8  # Activation precision


@dataclass
class AutomaticQuantizeOption(QuantizationOptions):
    use_cuda: bool = False
    suggestion_only: bool = False
    metric: SimilarityMetric = SimilarityMetric.SNR  # Quantization quality metrics
    threshold: Union[float, int] = 0  # Quantization quality threshold
    weight_precision: QuantizationPrecision = QuantizationPrecision.INT8  # Weight precision
    activation_precision: QuantizationPrecision = QuantizationPrecision.INT8  # Activation precision


@dataclass
class RequestQuantizeTask:
    input_model_id: str
    quantization_mode: QuantizationMode
    quantization_options: Union[QuantizationOptions, CustomQuantizeOption]
    input_layers: Optional[List[InputLayer]] = None

    def __post_init__(self):
        self.quantization_options = json.dumps(asdict(self.quantization_options))

        if self.input_layers:
            self.input_layers = json.dumps([asdict(input_layer) for input_layer in self.input_layers])



import json
from dataclasses import asdict, dataclass, field
from typing import Dict, List

from netspresso.enums.metadata import Status, TaskType


@dataclass
class InputShape:
    batch: int = 1
    channel: int = 3
    dimension: List[int] = field(default_factory=list)


@dataclass
class ModelInfo:
    task: str = ""
    model: str = ""
    dataset: str = ""
    input_shapes: List[InputShape] = field(default_factory=lambda: [InputShape()])


@dataclass
class TrainingInfo:
    epoch: int = 0
    batch_size: int = 0


@dataclass
class TrainerMetadata:
    status: Status = Status.IN_PROGRESS
    task_type: TaskType = TaskType.TRAIN
    model_info: ModelInfo = field(default_factory=ModelInfo)
    training_info: TrainingInfo = field(default_factory=TrainingInfo)
    logging_dir: str = ""
    best_fx_model_path: str = ""
    best_onnx_model_path: str = ""
    hparams: str = ""
    training_result: Dict = field(default_factory=dict)

    def asdict(self) -> Dict:
        _dict = json.loads(json.dumps(asdict(self)))
        return _dict

    def update_status(self, status: Status):
        self.status = status

    def update_model_info(self, task, model, dataset, input_shapes):
        self.model_info.task = task
        self.model_info.model = model
        self.model_info.dataset = dataset
        self.model_info.input_shapes = input_shapes

    def update_training_info(self, epoch, batch_size):
        self.training_info.epoch = epoch
        self.training_info.batch_size = batch_size

    def update_training_result(self, training_summary):
        self.training_result = training_summary

    def update_logging_dir(self, logging_dir):
        self.logging_dir = logging_dir

    def update_best_fx_model_path(self, best_fx_model_path):
        self.best_fx_model_path = best_fx_model_path

    def update_best_onnx_model_path(self, best_onnx_model_path):
        self.best_onnx_model_path = best_onnx_model_path

    def update_hparams(self, hparams):
        self.hparams = hparams



from dataclasses import asdict
from enum import Enum

from loguru import logger

from netspresso.clients.launcher.v2.interfaces import TaskInterface
from netspresso.clients.launcher.v2.schemas import (
    AuthorizationHeader,
    ResponseQuantizeDownloadModelUrlItem,
    ResponseQuantizeOptionItems,
    ResponseQuantizeStatusItem,
    ResponseQuantizeTaskItem,
    UploadFile,
)
from netspresso.clients.launcher.v2.schemas.common import UploadDataset
from netspresso.clients.launcher.v2.schemas.task.quantize.request_body import RequestQuantizeTask
from netspresso.clients.utils.requester import Requester
from netspresso.enums import LauncherTask


class QuantizeTaskAPI(TaskInterface):
    def __init__(self, url):
        self.task_type = LauncherTask.QUANTIZE.value
        self.base_url = url
        self.task_base_url = f"{self.base_url}/{self.task_type}/tasks"
        self.option_base_url = f"{self.base_url}/{self.task_type}/options"
        self.model_base_url = f"{self.base_url}/{self.task_type}/models"

    @staticmethod
    def custom_asdict_factory(data):
        def convert_value(obj):
            if isinstance(obj, Enum):
                return obj.value
            return obj

        return {k: convert_value(v) for k, v in data}

    def _request_to_quantizer(self, request_body, endpoint, headers, file):
        logger.info(f"Request_Body: {asdict(request_body)}")
        response = Requester().post_as_form(
            url=endpoint,
            request_body=asdict(request_body, dict_factory=self.custom_asdict_factory),
            headers=headers.to_dict(),
            binary=file.files if file else None,
        )

        return ResponseQuantizeTaskItem(**response.json())

    def start_plain_quantization(
        self, request_body: RequestQuantizeTask, headers: AuthorizationHeader, file: UploadDataset = None
    ) -> ResponseQuantizeTaskItem:
        endpoint = f"{self.task_base_url}"

        response = self._request_to_quantizer(request_body, endpoint, headers, file)

        return response

    def start_recommendation_precision(
        self, request_body: RequestQuantizeTask, headers: AuthorizationHeader, file: UploadDataset = None
    ) -> ResponseQuantizeTaskItem:
        endpoint = f"{self.task_base_url}/recommendation"

        response = self._request_to_quantizer(request_body, endpoint, headers, file)

        return response

    def start_custom_quantization(
        self, request_body: RequestQuantizeTask, headers: AuthorizationHeader, file: UploadDataset = None
    ) -> ResponseQuantizeTaskItem:
        endpoint = f"{self.task_base_url}/custom"

        response = self._request_to_quantizer(request_body, endpoint, headers, file)

        return response

    def start_auto_quantization(
        self, request_body: RequestQuantizeTask, headers: AuthorizationHeader, file: UploadDataset = None
    ) -> ResponseQuantizeTaskItem:
        endpoint = f"{self.task_base_url}/auto"

        response = self._request_to_quantizer(request_body, endpoint, headers, file)

        return response

    def cancel(self, headers: AuthorizationHeader, task_id: str) -> ResponseQuantizeTaskItem:
        endpoint = f"{self.task_base_url}/{task_id}/cancel"
        response = Requester().post_as_json(url=endpoint, request_body={}, headers=headers.to_dict())
        return ResponseQuantizeTaskItem(**response.json())

    def read(self, headers: AuthorizationHeader, task_id: str) -> ResponseQuantizeTaskItem:
        endpoint = f"{self.task_base_url}/{task_id}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseQuantizeTaskItem(**response.json())

    def status(self, headers: AuthorizationHeader, task_id: str) -> ResponseQuantizeStatusItem:
        endpoint = f"{self.task_base_url}/{task_id}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseQuantizeStatusItem(**response.json())

    def options(self, headers: AuthorizationHeader) -> ResponseQuantizeOptionItems:
        endpoint = f"{self.option_base_url}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseQuantizeOptionItems(**response.json())

    def get_download_url(
        self, headers: AuthorizationHeader, quantize_task_uuid: str
    ) -> ResponseQuantizeDownloadModelUrlItem:
        endpoint = f"{self.task_base_url}/{quantize_task_uuid}/results"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseQuantizeDownloadModelUrlItem(**response.json())

    def start(self, request_body, headers, file, endpoint):
        pass

    def options_by_model_framework(self, headers: AuthorizationHeader, model_framework: str):
        pass

    def delete(self, headers: AuthorizationHeader, task_id: str):
        pass



from .common import ComputeUnit
from .compile import CompileOptions, Extension, Framework, QuantizeFullType, Runtime
from .profile import (
    ContextErrorReportingOptionsLevel,
    ContextGpuPerformanceHint,
    ContextHtpPerformanceMode,
    DefaultGraphGpuPrecision,
    DefaultGraphHtpOptimizationType,
    DefaultGraphHtpPrecision,
    ExecutionMode,
    GpuInferencePreference,
    GpuInferencePriority,
    GraphOptimizationLevel,
    InferenceOptions,
    NnapiExecutionPreference,
    OnnxExecutionProviders,
    OnnxOptions,
    OnnxQnnHtpPerformanceMode,
    OnnxQnnOptions,
    Priority,
    ProfileOptions,
    QnnDspEncoding,
    QnnDspPerformanceMode,
    QnnGpuPerformanceMode,
    QnnGpuPrecision,
    QnnGraphPriority,
    QnnHtpOptimizationStrategy,
    QnnHtpPrecision,
    QnnLogLevel,
    QnnOptions,
    TfliteDelegates,
    TfliteGpuv2Options,
    TfliteNnapiOptions,
    TfliteOptions,
    TfliteQnnHtpPerformanceMode,
    TfliteQnnOptions,
)
from .quantize import QuantizeOptions, QuantizePrecision, RangeScheme

__all__ = [
    "CompileOptions",
    "ComputeUnit",
    "QuantizeFullType",
    "Runtime",
    "Framework",
    "Extension",
    "ContextErrorReportingOptionsLevel",
    "ContextGpuPerformanceHint",
    "ContextHtpPerformanceMode",
    "DefaultGraphGpuPrecision",
    "DefaultGraphHtpOptimizationType",
    "DefaultGraphHtpPrecision",
    "ExecutionMode",
    "GpuInferencePreference",
    "GpuInferencePriority",
    "GraphOptimizationLevel",
    "InferenceOptions",
    "NnapiExecutionPreference",
    "OnnxExecutionProviders",
    "OnnxOptions",
    "OnnxQnnHtpPerformanceMode",
    "OnnxQnnOptions",
    "Priority",
    "ProfileOptions",
    "QnnDspEncoding",
    "QnnDspPerformanceMode",
    "QnnGpuPerformanceMode",
    "QnnGpuPrecision",
    "QnnGraphPriority",
    "QnnHtpOptimizationStrategy",
    "QnnHtpPrecision",
    "QnnLogLevel",
    "QnnOptions",
    "TfliteDelegates",
    "TfliteGpuv2Options",
    "TfliteNnapiOptions",
    "TfliteOptions",
    "TfliteQnnHtpPerformanceMode",
    "TfliteQnnOptions",
    "QuantizeOptions",
    "QuantizePrecision",
    "RangeScheme",
]



from dataclasses import dataclass
from typing import Optional

from netspresso.enums.base import StrEnum
from netspresso.np_qai.options.common import CommonOptions


class Framework(StrEnum):
    PYTORCH = "pytorch"
    ONNX = "onnx"
    ONNXRUNTIME = "onnxruntime"
    AIMET = "aimet"
    TENSORFLOW = "tensorflow"
    TFLITE = "tensorflow_lite"
    COREML = "coreml"
    TENSORRT = "tensorrt"
    QNN = "qnn"


class Extension(StrEnum):
    ONNX = ".onnx"
    PT = ".pt"
    AIMET = ".aimet"
    H5 = ".h5"


class Runtime(StrEnum):
    TFLITE = "tflite"
    QNN_LIB_AARCH64_ANDROID = "qnn_lib_aarch64_android"
    QNN_CONTEXT_BINARY = "qnn_context_binary"
    ONNX = "onnx"
    PRECOMPILED_QNN_ONNX = "precompiled_qnn_onnx"


class QuantizeFullType(StrEnum):
    INT8 = "int8"
    INT16 = "int16"
    W8A16 = "w8a16"
    W4A8 = "w4a8"
    W4A16 = "w4a16"


class QuantizeWeightType(StrEnum):
    FP16 = "float16"


@dataclass
class CompileOptions(CommonOptions):
    """
    Compile options for the model.

    Note:
        For details, see `CompileOptions in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/api.html#compile-options>`_.
    """

    target_runtime: Optional[Runtime] = Runtime.TFLITE
    output_names: Optional[str] = None
    truncate_64bit_tensors: Optional[bool] = False
    truncate_64bit_io: Optional[bool] = False
    force_channel_last_input: Optional[str] = None
    force_channel_last_output: Optional[str] = None
    quantize_full_type: Optional[QuantizeFullType] = None
    quantize_weight_type: Optional[QuantizeWeightType] = None
    quantize_io: Optional[bool] = False
    quantize_io_type: Optional[str] = None
    qnn_graph_name: Optional[str] = None
    qnn_context_binary_vtcm: Optional[str] = None
    qnn_context_binary_optimization_level: Optional[int] = None

    def to_cli_string(self) -> str:
        args = []
        if self.compute_unit is not None:
            compute_units = ",".join([unit.name.lower() for unit in self.compute_unit])
            args.append(f"--compute_unit {compute_units}")
        if self.target_runtime is not None:
            args.append(f"--target_runtime {self.target_runtime}")
        if self.output_names is not None:
            output_names_str = ",".join(self.output_names.split())  # Split and join to handle spaces
            args.append(f'--output_names "{output_names_str}"')
        if self.truncate_64bit_tensors:
            args.append("--truncate_64bit_tensors")
        if self.truncate_64bit_io:
            args.append("--truncate_64bit_io")
        if self.force_channel_last_input is not None:
            args.append(f'--force_channel_last_input "{self.force_channel_last_input}"')
        if self.force_channel_last_output is not None:
            args.append(f'--force_channel_last_output "{self.force_channel_last_output}"')
        if self.quantize_full_type is not None:
            args.append(f"--quantize_full_type {self.quantize_full_type}")
        if self.quantize_weight_type is not None:
            args.append(f"--quantize_weight_type {self.quantize_weight_type}")
        if self.quantize_io:
            args.append("--quantize_io")
        if self.quantize_io_type is not None:
            args.append(f"--quantize_io_type {self.quantize_io_type}")
        if self.qnn_graph_name is not None:
            args.append(f"--qnn_graph_name {self.qnn_graph_name}")
        if self.qnn_context_binary_vtcm is not None:
            args.append(f"--qnn_context_binary_vtcm {self.qnn_context_binary_vtcm}")
        if self.qnn_context_binary_optimization_level is not None:
            args.append(f"--qnn_context_binary_optimization_level {self.qnn_context_binary_optimization_level}")

        return " ".join(args)



from pathlib import Path
from typing import List, Optional, Union

import qai_hub as hub
from loguru import logger
from qai_hub import JobStatus
from qai_hub.client import Dataset, Device, InferenceJob, ProfileJob, ProfileJobResult
from qai_hub.public_rest_api import DatasetEntries

from netspresso.analytics import netspresso_analytics
from netspresso.enums import Status
from netspresso.metadata.benchmarker import BenchmarkerMetadata
from netspresso.np_qai.base import NPQAIBase
from netspresso.np_qai.options import InferenceOptions, ProfileOptions
from netspresso.np_qai.options.common import normalize_device_name
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class NPQAIBenchmarker(NPQAIBase):
    def download_benchmark_results(self, job: ProfileJob, artifacts_dir: str) -> ProfileJobResult:
        """
        Downloads the benchmark results from a given ProfileJob.

        Args:
            job: The ProfileJob to download the results from.
            artifacts_dir: The directory to save the results to.

        Returns:
            ProfileJobResult: The benchmark results.

        Note:
            For details, see `download_results in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.ProfileJob.html#qai_hub.ProfileJob.download_results>`_.
        """
        results = job.download_results(artifacts_dir=artifacts_dir)

        return results

    def download_profile(self, job: ProfileJob):
        """
        Retrieves the profile data from the given ProfileJob.

        Args:
            job: The ProfileJob to download the profile from.

        Returns:
            The profile data.

        Note:
            For details, see `download_profile in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.ProfileJob.html#qai_hub.ProfileJob.download_profile>`_.
        """
        profile = job.download_profile()

        return profile

    def get_benchmark_task_status(self, benchmark_task_id: str) -> JobStatus:
        """
        Get the status of a benchmark task.

        Args:
            benchmark_task_id: The ID of the benchmark task to get the status of.

        Returns:
            JobStatus: The status of the benchmark task.

        Note:
            For details, see `JobStatus in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.JobStatus.html>`_.
        """
        job: ProfileJob = hub.get_job(benchmark_task_id)
        status = job.get_status()

        return status

    def update_benchmark_task(self, metadata: BenchmarkerMetadata) -> BenchmarkerMetadata:
        """
        Update the benchmark task.

        Args:
            metadata: The metadata of the benchmark task.

        Returns:
            BenchmarkerMetadata: The updated metadata of the benchmark task.
        """
        job: ProfileJob = hub.get_job(metadata.benchmark_task_info.benchmark_task_uuid)
        status = job.wait()

        if status.success:
            logger.info(f"{status.symbol} {status.state.name}")
            profile = self.download_profile(job=job)
            metadata.benchmark_result.latency = profile["execution_summary"]["estimated_inference_time"] / 1000
            metadata.benchmark_result.memory_footprint = profile["execution_summary"]["estimated_inference_peak_memory"]
            metadata.status = Status.COMPLETED
        elif status.failure:
            logger.info(f"{status.symbol} {status.state}: {status.message}")
            metadata.status = Status.ERROR

        folder_path = Path(metadata.input_model_path).parent
        file_path = folder_path / "benchmark.json"
        metadatas = []
        if FileHandler.check_exists(file_path):
            metadatas = MetadataHandler.load_json(file_path)

        for i, stored_metadata in enumerate(metadatas):
            if (
                stored_metadata.get("benchmark_task_info", {}).get("benchmark_task_uuid")
                == metadata.benchmark_task_info.benchmark_task_uuid
            ):
                metadatas[i] = metadata.asdict()
                break

        MetadataHandler.save_json(data=metadatas, folder_path=folder_path, file_name="benchmark")

        return metadata

    def benchmark_model(
        self,
        input_model_path: Union[str, Path],
        target_device_name: Union[Device, List[Device]],
        options: Union[ProfileOptions, str] = ProfileOptions(),
        job_name: Optional[str] = None,
        retry: bool = True,
    ) -> Union[BenchmarkerMetadata, List[BenchmarkerMetadata]]:
        """
        Benchmark a model on a device in the QAI hub.

        Args:
            input_model_path: The path to the input model.
            target_device_name: The device to benchmark the model on.
            options: The options to use for the benchmark.
            job_name: The name of the job.
            retry: Whether to retry the benchmark if it fails.

        Returns:
            Union[BenchmarkerMetadata, List[BenchmarkerMetadata]]: Returns a benchmarker metadata object if successful.

        Note:
            For details, see `submit_profile_job in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.submit_profile_job.html>`_.
        """
        netspresso_analytics.send_event(
            event_name="benchmark_model",
            event_params={
                "target_device_name": normalize_device_name(target_device_name),
                "compute_unit": options.normalize_compute_units(),
            },
        )

        FileHandler.check_input_model_path(input_model_path)

        folder_path = Path(input_model_path).parent
        file_name = "benchmark"

        metadatas = []
        file_path = folder_path / f"{file_name}.json"
        if FileHandler.check_exists(file_path):
            metadatas = MetadataHandler.load_json(file_path)

        metadata = BenchmarkerMetadata()
        metadata.input_model_path = Path(input_model_path).resolve().as_posix()
        metadata.benchmark_task_info.device_name = target_device_name.name
        metadata.benchmark_task_info.display_device_name = target_device_name.name
        metadatas.append(metadata.asdict())
        MetadataHandler.save_json(data=metadatas, folder_path=folder_path, file_name=file_name)

        try:
            model_type = hub.client._determine_model_type(model=input_model_path)
            framework = self.get_framework_by_model_type(model_type=model_type)
            display_framework = self.get_display_framework(framework)
            metadata.benchmark_task_info.framework = framework
            metadata.benchmark_task_info.display_framework = display_framework

            cli_string = options.to_cli_string() if isinstance(options, ProfileOptions) else options

            job: ProfileJob = hub.submit_profile_job(
                model=input_model_path,
                device=target_device_name,
                name=job_name,
                options=cli_string,
                retry=retry,
            )

            metadata.benchmark_task_info.benchmark_task_uuid = job.job_id

        except KeyboardInterrupt:
            metadata.status = Status.STOPPED

        metadatas[-1] = metadata.asdict()
        MetadataHandler.save_json(data=metadatas, folder_path=folder_path, file_name=file_name)

        return metadata

    def get_inference_task_status(self, inference_task_id: str) -> JobStatus:
        """
        Get the status of an inference task.

        Args:
            inference_task_id: The ID of the inference task to get the status of.

        Returns:
            JobStatus: The status of the inference task.

        Note:
            For details, see `JobStatus in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.JobStatus.html>`_.
        """
        job: InferenceJob = hub.get_job(inference_task_id)
        status = job.get_status()

        return status

    def inference_model(
        self,
        input_model_path: Union[str, Path],
        target_device_name: Union[Device, List[Device]],
        inputs: Union[Dataset, DatasetEntries, str],
        job_name: Optional[str] = None,
        options: Union[InferenceOptions, str] = InferenceOptions(),
        retry: bool = True,
    ) -> Union[InferenceJob, List[InferenceJob]]:
        """
        Inference a model on a device in the QAI hub.

        Args:
            input_model_path: The path to the input model.
            target_device_name: The device to benchmark the model on.
            inputs: The input data to use for the inference.
            job_name: The name of the job.
            options: The options to use for the inference.
            retry: Whether to retry the inference if it fails.

        Returns:
            Union[InferenceJob, List[InferenceJob]]: Returns an inference job object if successful.

        Note:
            For details, see `submit_inference_job in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.submit_inference_job.html>`_.
        """
        netspresso_analytics.send_event(
            event_name="benchmark_model",
            event_params={
                "target_device_name": normalize_device_name(target_device_name),
            },
        )

        cli_string = options.to_cli_string() if isinstance(options, InferenceOptions) else options

        job: InferenceJob = hub.submit_inference_job(
            model=input_model_path,
            device=target_device_name,
            inputs=inputs,
            name=job_name,
            options=cli_string,
            retry=retry,
        )

        return job



import platform

from pkg_resources import DistributionNotFound, get_distribution


def get_package_version(package_name):
    try:
        version = get_distribution(package_name).version
        return version
    except DistributionNotFound:
        return None


def get_installed_packages_version(package_keys):
    packages_version = {}
    packages_version["python"] = platform.python_version()
    packages_version["os"] = f"{platform.system()} {platform.version()}"

    for package_key in package_keys:
        packages_version[package_key] = get_package_version(package_key)

    return packages_version


def generate_env_string(package_keys):
    packages_version = get_installed_packages_version(package_keys)
    env_str = "; ".join(f"{key} {value}" for key, value in packages_version.items() if value is not None)

    return env_str


PACKAGE_KEYS = ["torch", "tensorflow", "tensorflow-gpu", "numpy"]
ENV_STR = generate_env_string(PACKAGE_KEYS)



import json
from dataclasses import asdict, dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.enums.metadata import Status, TaskType

from .common import TargetDevice


@dataclass
class InputShape:
    batch: int = 1
    channel: int = 3
    dimension: List[int] = field(default_factory=list)


@dataclass
class Model:
    size: int = 0
    flops: int = 0
    number_of_parameters: int = 0
    trainable_parameters: int = 0
    non_trainable_parameters: int = 0
    number_of_layers: Optional[int] = None
    model_id: str = ""


@dataclass
class ModelInfo:
    task: str = ""
    framework: str = ""
    input_shapes: List[InputShape] = field(default_factory=lambda: [InputShape()])


@dataclass
class CompressionInfo:
    method: str = ""
    ratio: float = 0.0
    options: Dict[str, Any] = None
    layers: List[Dict] = field(default_factory=list)


@dataclass
class Results:
    original_model: Model = field(default_factory=Model)
    compressed_model: Model = field(default_factory=Model)


@dataclass
class CompressorMetadata:
    status: Status = Status.IN_PROGRESS
    task_type: TaskType = TaskType.COMPRESS
    compressed_model_path: str = ""
    compressed_onnx_model_path: str = ""
    results: Results = field(default_factory=Results)
    model_info: ModelInfo = field(default_factory=ModelInfo)
    compression_info: CompressionInfo = field(default_factory=CompressionInfo)
    available_devices: List[TargetDevice] = field(default_factory=list)

    def asdict(self) -> Dict:
        _dict = json.loads(json.dumps(asdict(self)))
        return _dict

    def update_status(self, status: Status):
        self.status = status

    def update_model_info(self, task, framework, input_shapes):
        self.model_info.task = task
        self.model_info.framework = framework
        self.model_info.input_shapes = input_shapes

    def update_compression_info(self, method, options, layers, ratio=0.0):
        self.compression_info.method = method
        self.compression_info.ratio = ratio
        self.compression_info.options = options
        self.compression_info.layers = layers

    def update_compressed_model_path(self, compressed_model_path):
        self.compressed_model_path = compressed_model_path

    def update_compressed_onnx_model_path(self, compressed_onnx_model_path):
        self.compressed_onnx_model_path = compressed_onnx_model_path

    def update_results(self, model, compressed_model):
        def update_model_fields(target, source):
            target.size = source.model_size
            target.flops = source.flops
            target.number_of_parameters = source.trainable_parameters + source.non_trainable_parameters
            target.trainable_parameters = source.trainable_parameters
            target.non_trainable_parameters = source.non_trainable_parameters
            target.number_of_layers = source.number_of_layers if source.number_of_layers != 0 else None
            target.model_id = source.model_id

        update_model_fields(self.results.original_model, model)
        update_model_fields(self.results.compressed_model, compressed_model)

    def update_available_devices(self, available_devices):
        self.available_devices = [
            TargetDevice(
                display_name=device.display_name,
                display_brand_name=device.display_brand_name,
                device_name=device.device_name,
                software_version=device.software_version,
                software_version_display_name=device.software_version_display_name,
                hardware_type=device.hardware_type,
            )
            for device in available_devices
        ]



import os
import tarfile


def split_tar_file(input_tar_path, output_dir, max_split_size=200 * 1024 * 1024):
    os.makedirs(output_dir, exist_ok=True)

    with tarfile.open(input_tar_path, "r") as original_tar:
        members = original_tar.getmembers()
        current_split_size = 0
        current_split_number = 0
        current_split_name = os.path.join(output_dir, f"smaller_file_{current_split_number}.tar")

        with tarfile.open(current_split_name, "w") as split_tar:
            for member in members:
                if current_split_size + member.size <= max_split_size:
                    split_tar.addfile(member, original_tar.extractfile(member))
                    current_split_size += member.size
                else:
                    split_tar.close()
                    current_split_number += 1
                    current_split_name = os.path.join(output_dir, f"smaller_file_{current_split_number}.tar")
                    current_split_size = 0
                    split_tar = tarfile.open(current_split_name, "w")  # Open a new split tar archive
                    split_tar.addfile(member, original_tar.extractfile(member))
                    current_split_size += member.size



from loguru import logger

from netspresso.clients.auth import TokenHandler, auth_client
from netspresso.enums import ServiceCredit, ServiceTask, Status
from netspresso.exceptions.common import NotEnoughCreditException
from netspresso.metadata.common import BaseMetadata


class NetsPressoBase:
    def __init__(self, token_handler: TokenHandler) -> None:
        self.token_handler = token_handler
        self.auth_client = auth_client

    def check_credit_balance(self, service_task: ServiceTask):
        current_credit = self.auth_client.get_credit(
            access_token=self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
        )
        service_credit = ServiceCredit.get_credit(service_task)
        service_task_name = service_task.name.replace("_", " ").lower()
        if current_credit < service_credit:
            logger.error(
                f"Insufficient balance: {current_credit} credits available, but {service_credit} credits required for {service_task_name} task."
            )
            raise NotEnoughCreditException(current_credit, service_credit, service_task_name)

    def print_remaining_credit(self, service_task):
        if self.auth_client.is_cloud():
            self.token_handler.validate_token()
            service_credit = ServiceCredit.get_credit(service_task)
            remaining_credit = self.auth_client.get_credit(
                self.token_handler.tokens.access_token, verify_ssl=self.token_handler.verify_ssl
            )
            logger.info(f"{service_credit} credits have been consumed. Remaining Credit: {remaining_credit}")

    def validate_token_and_check_credit(self, service_task: ServiceTask):
        self.token_handler.validate_token()
        self.check_credit_balance(service_task=service_task)

    def handle_error(self, metadata: BaseMetadata, task_name: ServiceTask, error_message: str):
        metadata.status = Status.ERROR
        metadata.update_message(exception_detail=error_message)
        logger.error(f"{task_name} task failed due to an error: {error_message}")

        return metadata

    def handle_stop(self, metadata: BaseMetadata, task_name: ServiceTask):
        metadata.status = Status.STOPPED
        logger.error(f"{task_name} task was interrupted by the user.")

        return metadata



from netspresso.trainer.augmentations.augmentation import (
    AugmentationConfig,
    CenterCrop,
    ClassificationAugmentationConfig,
    ColorJitter,
    DetectionAugmentationConfig,
    HSVJitter,
    Mixing,
    MosaicDetection,
    Normalize,
    Pad,
    PoseTopDownAffine,
    RandomCrop,
    RandomCutmix,
    RandomErasing,
    RandomHorizontalFlip,
    RandomMixup,
    RandomResize,
    RandomResize2,
    RandomResizedCrop,
    RandomVerticalFlip,
    Resize,
    SegmentationAugmentationConfig,
    ToTensor,
    Transform,
    TrivialAugmentWide,
)

AUGMENTATION_CONFIG_TYPE = {
    "classification": ClassificationAugmentationConfig,
    "detection": DetectionAugmentationConfig,
    "segmentation": SegmentationAugmentationConfig,
}


__all__ = [
    "CenterCrop",
    "HSVJitter",
    "Mixing",
    "MosaicDetection",
    "PoseTopDownAffine",
    "RandomErasing",
    "RandomResize",
    "RandomResize2",
    "ColorJitter",
    "Pad",
    "RandomCrop",
    "RandomResizedCrop",
    "RandomHorizontalFlip",
    "RandomVerticalFlip",
    "Resize",
    "TrivialAugmentWide",
    "RandomMixup",
    "RandomCutmix",
    "Transform",
    "AugmentationConfig",
    "AUGMENTATION_CONFIG_TYPE",
    "ToTensor",
    "Normalize",
]



from dataclasses import dataclass, field
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

from omegaconf import MISSING


@dataclass
class ArchitectureConfig:
    full: Optional[Dict[str, Any]] = None
    backbone: Optional[Dict[str, Any]] = None
    neck: Optional[Dict[str, Any]] = None
    head: Optional[Dict[str, Any]] = None

    def __post_init__(self):
        assert bool(self.full) != bool(self.backbone), "Only one of full or backbone should be given."


@dataclass
class CheckpointConfig:
    use_pretrained: bool = True
    load_head: bool = False
    path: Optional[Union[Path, str]] = None
    fx_model_path: Optional[Union[Path, str]] = None
    optimizer_path: Optional[Union[Path, str]] = None


@dataclass
class ModelConfig:
    task: str = MISSING
    name: str = MISSING
    checkpoint: CheckpointConfig = field(default_factory=lambda: CheckpointConfig())
    freeze_backbone: bool = False
    architecture: ArchitectureConfig = field(default_factory=lambda: ArchitectureConfig())
    postprocessor: Optional[Dict[str, Any]] = None
    losses: Optional[List[Dict[str, Any]]] = None



import dataclasses
from dataclasses import dataclass, field
from typing import List, Optional

from netspresso.clients.launcher.v2.schemas import (
    InputLayer,
    ModelOption,
    ResponseItem,
    ResponseItems,
    TaskInfo,
    TaskOption,
)
from netspresso.clients.launcher.v2.schemas.task.common import TaskStatusInfo
from netspresso.enums import Framework, TaskStatusForDisplay
from netspresso.metadata.converter import ConvertInfo


@dataclass
class ConvertTask:
    convert_task_id: str
    input_model_id: str
    output_model_id: str
    input_layer: InputLayer = field(default_factory=InputLayer)
    status: TaskStatusForDisplay = ""
    error_log: Optional[dict] = None
    convert_task_option: Optional[TaskOption] = field(default_factory=TaskOption)

    def __init__(self, **kwargs):
        names = {f.name for f in dataclasses.fields(self)}
        for k, v in kwargs.items():
            if k in names:
                setattr(self, k, v)

        self.input_layer = InputLayer(**self.input_layer)
        self.convert_task_option = TaskOption(**self.convert_task_option)

    def to(self, model_file_name: str) -> ConvertInfo:
        device_info = self.convert_task_option.target_device

        convert_info = ConvertInfo()
        convert_info.convert_task_uuid = self.convert_task_id
        convert_info.framework = self.convert_task_option.framework
        convert_info.display_framework = self.convert_task_option.display_framework
        convert_info.input_model_uuid = self.input_model_id
        convert_info.output_model_uuid = self.output_model_id
        convert_info.model_file_name = model_file_name

        convert_info.device_name = device_info.device_name
        convert_info.display_device_name = device_info.display_device_name
        convert_info.display_brand_name = device_info.display_brand_name

        convert_info.data_type = device_info.data_type

        convert_info.software_version = device_info.software_version
        convert_info.display_software_version = device_info.display_software_version

        return convert_info


@dataclass
class ResponseConvertTaskItem(ResponseItem):
    data: Optional[ConvertTask] = field(default_factory=dict)

    def __post_init__(self):
        self.data = ConvertTask(**self.data)


@dataclass
class ConvertOption:
    option_name: str
    display_option: str
    framework: Framework
    device: TaskInfo

    def __init__(self, **kwargs):
        names = {f.name for f in dataclasses.fields(self)}
        for k, v in kwargs.items():
            if k in names:
                setattr(self, k, v)

        self.device = TaskInfo(**self.device)


@dataclass
class ResponseConvertOptionItems(ResponseItems):
    data: List[Optional[ConvertOption]] = field(default_factory=list)

    def __post_init__(self):
        self.data = [ConvertOption(**item) for item in self.data]


@dataclass
class ResponseConvertStatusItem(ResponseItem):
    data: TaskStatusInfo = field(default_factory=TaskStatusInfo)

    def __post_init__(self):
        self.data = TaskStatusInfo(**self.data)


@dataclass
class DownloadModelUrl:
    ai_model_id: str
    presigned_download_url: str


@dataclass
class ResponseConvertDownloadModelUrlItem(ResponseItem):
    data: Optional[DownloadModelUrl] = field(default_factory=dict)

    def __post_init__(self):
        self.data = DownloadModelUrl(**self.data)


@dataclass
class ResponseConvertFrameworkOptionItems(ResponseItems):
    data: List[Optional[ModelOption]] = field(default_factory=list)

    def __post_init__(self):
        self.data = [ModelOption(**item) for item in self.data]



import json
from pathlib import Path
from typing import Any, Dict, List, Union

from loguru import logger

from netspresso.metadata.benchmarker import BenchmarkerMetadata
from netspresso.metadata.common import BaseMetadata


class MetadataHandler:
    @staticmethod
    def save_metadata(data: BaseMetadata, folder_path: str, file_name: str = "metadata") -> None:
        """Save metadata to a JSON file.

        Args:
            data (BaseMetadata): The metadata object to be saved. It should be an instance of BaseMetadata or its subclass.
            folder_path (str): The directory path where the JSON file will be saved.
            file_name (str): The name of the JSON file to be created (without extension). Defaults to "metadata".

        Returns:
            None: This function does not return any value.

        Notes:
            The data is converted to a dictionary using the `asdict` method before saving.
        """
        MetadataHandler.save_json(data.asdict(), folder_path, file_name)

    @staticmethod
    def save_benchmark_result(data: List[BenchmarkerMetadata], folder_path: str, file_name: str = "benchmark") -> None:
        """Save a list of benchmark metadata objects to a JSON file.

        Args:
            data (List[BenchmarkerMetadata]): A list of BenchmarkerMetadata objects to be saved. Each object is converted to a dictionary using the `asdict` method.
            folder_path (str): The directory path where the JSON file will be saved.
            file_name (str): The name of the JSON file to be created (without extension). Defaults to "benchmark".

        Returns:
            None: This function does not return any value.

        Notes:
            Each item in the `data` list is checked to ensure it is an instance of BenchmarkerMetadata. If it is, it is converted to a dictionary before saving.
        """
        data = [_data.asdict() if isinstance(_data, BenchmarkerMetadata) else _data for _data in data]
        MetadataHandler.save_json(data, folder_path, file_name)

    def save_json(data: Union[Dict, List[Dict]], folder_path: str, file_name: str) -> None:
        """Save data to a JSON file.

        Args:
            data (Union[Dict, List[Dict]]): The data to be saved. This can be a dictionary or a list of dictionaries.
            folder_path (str): The directory path where the JSON file will be saved.
            file_name (str): The name of the JSON file to be created (without extension).

        Returns:
            None: This function does not return any value.

        Notes:
            The data is written to a JSON file with indentation for readability. The file is saved in the specified directory with the given name.
        """
        file_path = Path(folder_path) / f"{file_name}.json"

        with open(file_path, "w") as json_file:
            json.dump(data, json_file, indent=4)
        logger.info(f"JSON file saved at {file_path.resolve()}")

    @staticmethod
    def load_json(file_path: str) -> Dict[str, Any]:
        """Load JSON data from a file.

        Args:
            file_path (str): The path to the JSON file.

        Returns:
            dict: Loaded dictionary data.
        """
        with open(file_path, "r") as json_file:
            data = json.load(json_file)
        return data



from .request_body import RequestQuantizeTask
from .response_body import (
    ResponseQuantizeDownloadModelUrlItem,
    ResponseQuantizeOptionItems,
    ResponseQuantizeStatusItem,
    ResponseQuantizeTaskItem,
)

__all__ = [
    "RequestQuantizeTask",
    "ResponseQuantizeDownloadModelUrlItem",
    "ResponseQuantizeOptionItems",
    "ResponseQuantizeStatusItem",
    "ResponseQuantizeTaskItem",
]



import numpy as np


class SegmentationPostprocessor:
    def __init__(self):
        pass

    def __call__(self, outputs, original_shape):
        pred = outputs["pred"]
        H, W = original_shape[-2:]

        # Upsample logits to the images' original size using bilinear interpolation
        pred_upsampled = np.zeros((pred.shape[0], pred.shape[1], H, W))

        for i in range(pred.shape[0]):  # Iterate over the batch
            for j in range(pred.shape[1]):  # Iterate over the channels (classes)
                pred_upsampled[i, j] = self.bilinear_interpolate(pred[i, j], (H, W))

        # Take the argmax over the channel dimension to get the predicted class for each pixel
        pred_classes = np.argmax(pred_upsampled, axis=1)

        return pred_classes

    def bilinear_interpolate(self, img, new_shape):
        H, W = new_shape
        h_old, w_old = img.shape

        # Calculate the ratios
        row_ratio, col_ratio = h_old / H, w_old / W

        # Create a meshgrid of the target image size
        row_coords = np.arange(H) * row_ratio
        col_coords = np.arange(W) * col_ratio

        row_floor = np.floor(row_coords).astype(np.int32)
        col_floor = np.floor(col_coords).astype(np.int32)

        row_ceil = np.ceil(row_coords).clip(max=h_old - 1).astype(np.int32)
        col_ceil = np.ceil(col_coords).clip(max=w_old - 1).astype(np.int32)

        # Get the values of the four surrounding pixels for interpolation
        top_left = img[row_floor[:, None], col_floor]
        top_right = img[row_floor[:, None], col_ceil]
        bottom_left = img[row_ceil[:, None], col_floor]
        bottom_right = img[row_ceil[:, None], col_ceil]

        # Calculate the fractional part of the coordinates
        row_frac = row_coords - row_floor
        col_frac = col_coords - col_floor

        # Perform bilinear interpolation
        top = top_left * (1 - col_frac) + top_right * col_frac
        bottom = bottom_left * (1 - col_frac) + bottom_right * col_frac
        interpolated = top * (1 - row_frac[:, None]) + bottom * row_frac[:, None]

        return interpolated



import json
from dataclasses import asdict, dataclass, field
from typing import Optional

from netspresso.clients.launcher.v2.schemas import InputLayer
from netspresso.enums import DataType, DeviceName, Framework, SoftwareVersion


@dataclass
class RequestConvert:
    input_model_id: str
    target_framework: Framework
    target_device_name: DeviceName
    data_type: Optional[DataType] = None
    input_layer: Optional[InputLayer] = field(default_factory=InputLayer)
    software_version: Optional[SoftwareVersion] = ""

    def __post_init__(self):
        if self.input_layer:
            self.input_layer = json.dumps(asdict(self.input_layer))



from dataclasses import dataclass
from typing import Optional

from netspresso.clients.launcher.v2.schemas import InputLayer
from netspresso.enums import DataType, DeviceName, HardwareType, SoftwareVersion


@dataclass
class RequestBenchmark:
    input_model_id: str
    target_device_name: DeviceName
    data_type: Optional[DataType] = None
    software_version: Optional[SoftwareVersion] = ""
    hardware_type: Optional[HardwareType] = None
    input_layer: Optional[InputLayer] = None



from typing import Literal

from netspresso.enums.base import StrEnum


class Module(StrEnum):
    CONVERT = "CONVERT"
    BENCHMARK = "BENCHMARK"

    @classmethod
    def create_literal(cls):
        return Literal["CONVERT", "BENCHMARK"]



import time
from pathlib import Path
from typing import Optional, Union

from loguru import logger

from netspresso.analytics import netspresso_analytics
from netspresso.base import NetsPressoBase
from netspresso.clients.auth import TokenHandler
from netspresso.clients.auth.response_body import UserResponse
from netspresso.clients.launcher import launcher_client_v2
from netspresso.clients.launcher.v2.schemas.task.benchmark.response_body import BenchmarkTask
from netspresso.enums import Status, TaskStatusForDisplay
from netspresso.enums.credit import ServiceTask
from netspresso.enums.device import DeviceName, HardwareType, SoftwareVersion
from netspresso.enums.model import DataType
from netspresso.metadata.benchmarker import BenchmarkerMetadata
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class BenchmarkerV2(NetsPressoBase):
    def __init__(self, token_handler: TokenHandler, user_info: UserResponse) -> None:
        """Initialize the Benchmarker."""

        super().__init__(token_handler)
        self.user_info = user_info

    def get_data_type(self, input_model_dir):
        metadata_path = input_model_dir / "metadata.json"
        try:
            if metadata_path.exists():
                metadata = MetadataHandler.load_json(metadata_path)
                convert_task_info = metadata.get("convert_task_info", {})
                return convert_task_info.get("data_type", DataType.FP32)
        except (FileNotFoundError, ValueError, KeyError) as e:
            print(f"Error loading metadata: {e}")

        return DataType.FP32

    def initialize_metadata(self, input_model_path: str):
        def create_metadata_with_status(status, error_message=None):
            metadata = BenchmarkerMetadata()
            metadata.status = status
            if error_message:
                logger.error(error_message)
            return metadata

        try:
            metadata = BenchmarkerMetadata()
        except Exception as e:
            error_message = f"An unexpected error occurred during metadata initialization: {e}"
            metadata = create_metadata_with_status(Status.ERROR, error_message)
        except KeyboardInterrupt:
            warning_message = "Benchmark task was interrupted by the user."
            metadata = create_metadata_with_status(Status.STOPPED, warning_message)
        finally:
            # Load existing metadata if available
            metadatas = []
            output_dir = Path(input_model_path).parent
            file_path = output_dir / "benchmark.json"
            if FileHandler.check_exists(file_path):
                metadatas = MetadataHandler.load_json(file_path)

            metadata.input_model_path = Path(input_model_path).resolve().as_posix()
            metadatas.append(metadata)
            MetadataHandler.save_benchmark_result(data=metadatas, folder_path=output_dir)

        return metadatas

    def benchmark_model(
        self,
        input_model_path: str,
        target_device_name: DeviceName,
        target_software_version: Optional[Union[str, SoftwareVersion]] = None,
        target_hardware_type: Optional[Union[str, HardwareType]] = None,
        wait_until_done: bool = True,
        sleep_interval: int = 30,
    ) -> BenchmarkerMetadata:
        """Benchmark the specified model on the specified device.

        Args:
            input_model_path (str): The file path where the model is located.
            target_device_name (DeviceName): Target device name.
            target_software_version (Union[str, SoftwareVersion], optional): Target software version. Required if target_device_name is one of the Jetson devices.
            target_hardware_type (Union[str, HardwareType], optional): Hardware type. Acceleration options for processing the model inference.
            wait_until_done (bool): If True, wait for the benchmark result before returning the function.
                                If False, request the benchmark and return the function immediately.

        Raises:
            e: If an error occurs during the benchmarking of the model.

        Returns:
            BenchmarkerMetadata: Benchmark metadata.
        """

        netspresso_analytics.send_event(
            event_name="benchmark_model",
            event_params={
                "target_device_name": target_device_name,
                "target_software_version": target_software_version,
                "target_hardware_type": target_hardware_type,
            },
        )

        FileHandler.check_input_model_path(input_model_path)
        metadatas = self.initialize_metadata(input_model_path=input_model_path)

        try:
            metadata: BenchmarkerMetadata = metadatas[-1]
            output_dir = Path(input_model_path).parent
            if metadata.status in [Status.ERROR, Status.STOPPED]:
                return metadata

            self.validate_token_and_check_credit(service_task=ServiceTask.MODEL_BENCHMARK)

            # Get presigned_model_upload_url
            presigned_url_response = launcher_client_v2.benchmarker.presigned_model_upload_url(
                access_token=self.token_handler.tokens.access_token,
                input_model_path=input_model_path,
            )

            # Upload model_file
            launcher_client_v2.benchmarker.upload_model_file(
                access_token=self.token_handler.tokens.access_token,
                input_model_path=input_model_path,
                presigned_upload_url=presigned_url_response.data.presigned_upload_url,
            )

            # Validate model_file
            validate_model_response = launcher_client_v2.benchmarker.validate_model_file(
                access_token=self.token_handler.tokens.access_token,
                input_model_path=input_model_path,
                ai_model_id=presigned_url_response.data.ai_model_id,
            )

            # Start benchmark task
            benchmark_response = launcher_client_v2.benchmarker.start_task(
                access_token=self.token_handler.tokens.access_token,
                input_model_id=presigned_url_response.data.ai_model_id,
                data_type=validate_model_response.data.detail.data_type,
                target_device_name=target_device_name,
                hardware_type=target_hardware_type,
                input_layer=validate_model_response.data.detail.input_layers[0],
                software_version=target_software_version,
            )

            metadata.benchmark_task_info = benchmark_response.data.to()
            metadata.benchmark_task_info.data_type = self.get_data_type(output_dir)
            MetadataHandler.save_benchmark_result(data=metadatas, folder_path=output_dir)

            if wait_until_done:
                while True:
                    self.token_handler.validate_token()
                    benchmark_response = launcher_client_v2.benchmarker.read_task(
                        access_token=self.token_handler.tokens.access_token,
                        task_id=benchmark_response.data.benchmark_task_id,
                    )
                    if benchmark_response.data.status in [
                        TaskStatusForDisplay.FINISHED,
                        TaskStatusForDisplay.ERROR,
                        TaskStatusForDisplay.TIMEOUT,
                    ]:
                        break

                    time.sleep(sleep_interval)

            if benchmark_response.data.status == TaskStatusForDisplay.FINISHED:
                self.print_remaining_credit(service_task=ServiceTask.MODEL_BENCHMARK)
                metadata.status = Status.COMPLETED
                metadata.benchmark_result = benchmark_response.data.benchmark_result.to(
                    file_size=validate_model_response.data.file_size_in_mb
                )
                logger.info("Benchmark task was completed successfully.")
            else:
                metadata = self.handle_error(metadata, ServiceTask.MODEL_BENCHMARK, benchmark_response.data.error_log)

        except Exception as e:
            metadata = self.handle_error(metadata, ServiceTask.MODEL_BENCHMARK, e.args[0])
        except KeyboardInterrupt:
            metadata = self.handle_stop(metadata, ServiceTask.MODEL_BENCHMARK)
        finally:
            metadatas[-1] = metadata
            MetadataHandler.save_benchmark_result(data=metadatas, folder_path=output_dir)

        return metadata

    def get_benchmark_task(self, benchmark_task_id: str) -> BenchmarkTask:
        """Get information about the specified benchmark task using the benchmark task UUID.

        Args:
            benchmark_task_id (str): Benchmark task UUID of the benchmark task.

        Raises:
            e: If an error occurs while retrieving information about the benchmark task.

        Returns:
            BenchmarkTask: Model benchmark task object.
        """

        self.token_handler.validate_token()

        response = launcher_client_v2.benchmarker.read_task(
            access_token=self.token_handler.tokens.access_token,
            task_id=benchmark_task_id,
        )
        return response.data

    def cancel_benchmark_task(self, benchmark_task_id: str) -> BenchmarkTask:
        """Cancel the benchmark task with given benchmark task uuid.

        Args:
            benchmark_task_id (str): Benchmark task UUID of the benchmark task.

        Raises:
            e: If an error occurs during the task cancel.

        Returns:
            BenchmarkTask: Model benchmark task dictionary.
        """

        self.token_handler.validate_token()

        response = launcher_client_v2.benchmarker.cancel_task(
            access_token=self.token_handler.tokens.access_token,
            task_id=benchmark_task_id,
        )
        return response.data



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.trainer.models.base import ArchitectureConfig, CheckpointConfig, ModelConfig


@dataclass
class CSPDarkNetXArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "cspdarknet",
            "params": {
                "depthwise": False,
                "dep_mul": 1.33,
                "wid_mul": 1.25,
                "act_type": "silu",
            },
            "stage_params": None,
        }
    )


@dataclass
class CSPDarkNetLArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "cspdarknet",
            "params": {
                "depthwise": False,
                "dep_mul": 1.0,
                "wid_mul": 1.0,
                "act_type": "silu",
            },
            "stage_params": None,
        }
    )


@dataclass
class CSPDarkNetMArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "cspdarknet",
            "params": {
                "depthwise": False,
                "dep_mul": 0.67,
                "wid_mul": 0.75,
                "act_type": "silu",
            },
            "stage_params": None,
        }
    )


@dataclass
class CSPDarkNetSArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "cspdarknet",
            "params": {
                "depthwise": False,
                "dep_mul": 0.33,
                "wid_mul": 0.5,
                "act_type": "silu",
            },
            "stage_params": None,
        }
    )


@dataclass
class CSPDarkNetNanoArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "cspdarknet",
            "params": {
                "depthwise": True,
                "dep_mul": 0.33,
                "wid_mul": 0.25,
                "act_type": "silu",
            },
            "stage_params": None,
        }
    )


@dataclass
class CSPDarkNetTinyArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "cspdarknet",
            "params": {
                "depthwise": False,
                "dep_mul": 0.33,
                "wid_mul": 0.375,
                "act_type": "silu",
            },
            "stage_params": None,
        }
    )


@dataclass
class DetectionYoloXXModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "yolox_x"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: CSPDarkNetXArchitectureConfig(
            neck={
                "name": "yolopafpn",
                "params": {
                    "depthwise": False,
                    "dep_mul": 1.33,
                    "act_type": "silu",
                },
            },
            head={
                "name": "anchor_free_decoupled_head",
                "params": {
                    "depthwise": False,
                    "act_type": "silu",
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "score_thresh": 0.01,
                # postprocessor - nms
                "nms_thresh": 0.65,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "yolox_loss", "weight": None, "l1_activate_epoch": 1}]
    )


@dataclass
class DetectionYoloXLModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "yolox_l"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: CSPDarkNetLArchitectureConfig(
            neck={
                "name": "yolopafpn",
                "params": {
                    "depthwise": False,
                    "dep_mul": 1.0,
                    "act_type": "silu",
                },
            },
            head={
                "name": "anchor_free_decoupled_head",
                "params": {
                    "depthwise": False,
                    "act_type": "silu",
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "score_thresh": 0.01,
                # postprocessor - nms
                "nms_thresh": 0.65,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "yolox_loss", "weight": None, "l1_activate_epoch": 1}]
    )


@dataclass
class DetectionYoloXMModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "yolox_m"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: CSPDarkNetMArchitectureConfig(
            neck={
                "name": "yolopafpn",
                "params": {
                    "depthwise": False,
                    "dep_mul": 0.67,
                    "act_type": "silu",
                },
            },
            head={
                "name": "anchor_free_decoupled_head",
                "params": {
                    "depthwise": False,
                    "act_type": "silu",
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "score_thresh": 0.01,
                # postprocessor - nms
                "nms_thresh": 0.65,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "yolox_loss", "weight": None, "l1_activate_epoch": 1}]
    )


@dataclass
class DetectionYoloXSModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "yolox_s"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: CSPDarkNetSArchitectureConfig(
            neck={
                "name": "yolopafpn",
                "params": {
                    "depthwise": False,
                    "dep_mul": 0.33,
                    "act_type": "silu",
                },
            },
            head={
                "name": "anchor_free_decoupled_head",
                "params": {
                    "depthwise": False,
                    "act_type": "silu",
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "score_thresh": 0.01,
                # postprocessor - nms
                "nms_thresh": 0.65,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "yolox_loss", "weight": None, "l1_activate_epoch": 1}]
    )


@dataclass
class DetectionYoloXNanoModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "yolox_nano"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: CSPDarkNetNanoArchitectureConfig(
            neck={
                "name": "yolopafpn",
                "params": {
                    "depthwise": True,
                    "dep_mul": 0.33,
                    "act_type": "silu",
                },
            },
            head={
                "name": "anchor_free_decoupled_head",
                "params": {
                    "depthwise": True,
                    "act_type": "silu",
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "score_thresh": 0.01,
                # postprocessor - nms
                "nms_thresh": 0.65,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "yolox_loss", "weight": None, "l1_activate_epoch": 1}]
    )


@dataclass
class DetectionYoloXTinyModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "yolox_tiny"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: CSPDarkNetTinyArchitectureConfig(
            neck={
                "name": "yolopafpn",
                "params": {
                    "depthwise": False,
                    "dep_mul": 0.375,
                    "act_type": "silu",
                },
            },
            head={
                "name": "anchor_free_decoupled_head",
                "params": {
                    "depthwise": False,
                    "act_type": "silu",
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "score_thresh": 0.01,
                # postprocessor - nms
                "nms_thresh": 0.65,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "yolox_loss", "weight": None, "l1_activate_epoch": 1}]
    )



import json
from dataclasses import asdict, dataclass, field
from typing import Dict, List, Optional

from netspresso.enums.device import DeviceName, HardwareType, SoftwareVersion
from netspresso.enums.metadata import Status
from netspresso.enums.model import DataType, Framework


@dataclass
class InputShape:
    batch: int = 1
    channel: int = 3
    dimension: List[int] = field(default_factory=list)


@dataclass
class ModelInfo:
    data_type: DataType = ""
    framework: Framework = ""
    input_shapes: List[InputShape] = field(default_factory=list)


@dataclass
class SoftwareVersions:
    software_version: SoftwareVersion = ""
    display_software_versions: str = ""


@dataclass
class DeviceInfo:
    device_name: DeviceName = ""
    display_device_name: str = ""
    display_brand_name: str = ""
    software_versions: List[SoftwareVersions] = field(default_factory=list)
    data_types: List[DataType] = field(default_factory=list)
    hardware_types: List[HardwareType] = field(default_factory=list)


@dataclass
class AvailableOption:
    framework: Framework = ""
    display_framework: str = ""
    devices: List[DeviceInfo] = field(default_factory=list)


@dataclass
class LinkInfo:
    type: str
    value: str


@dataclass
class AdditionalData:
    origin: Optional[str] = ""
    error_log: Optional[str] = ""
    link: Optional[LinkInfo] = None


@dataclass
class ExceptionDetail:
    data: Optional[AdditionalData] = field(default_factory=AdditionalData)
    error_code: Optional[str] = ""
    name: Optional[str] = ""
    message: Optional[str] = ""


@dataclass
class BaseMetadata:
    status: Status = Status.IN_PROGRESS
    error_detail: ExceptionDetail = field(default_factory=ExceptionDetail)

    def asdict(self) -> Dict:
        _dict = json.loads(json.dumps(asdict(self)))
        return _dict

    def update_message(self, exception_detail):
        if isinstance(exception_detail, str):
            self.error_detail.message = exception_detail
        else:
            self.error_detail = ExceptionDetail(**exception_detail)

    def update_status(self, status: Status):
        self.status = status



import json
import os
import uuid
from typing import Any, Dict, Optional

import pkg_resources
import requests

# Debug mode controlled by environment variable
_DEBUG = os.getenv("GA_DEBUG_ANALYTICS")


def disable_request_logs():
    """Context manager to temporarily disable request logging"""
    import logging

    class LoggingFilter(logging.Filter):
        def filter(self, record):
            return False

    root_logger = logging.getLogger()
    request_logger = logging.getLogger("requests")
    urllib3_logger = logging.getLogger("urllib3")
    previous_level = root_logger.level
    previous_filters = {
        "root": list(root_logger.filters),
        "requests": list(request_logger.filters),
        "urllib3": list(urllib3_logger.filters),
    }
    log_filter = LoggingFilter()

    class DisableRequestLogs:
        def __enter__(self):
            if previous_level > logging.INFO:
                root_logger.setLevel(logging.INFO)
            root_logger.addFilter(log_filter)
            request_logger.addFilter(log_filter)
            urllib3_logger.addFilter(log_filter)
            return self

        def __exit__(self, exc_type, exc_val, exc_tb):
            root_logger.setLevel(previous_level)
            root_logger.filters = previous_filters["root"]
            request_logger.filters = previous_filters["requests"]
            urllib3_logger.filters = previous_filters["urllib3"]

    return DisableRequestLogs()


def is_gdpr_country() -> bool:
    """
    Check if the current IP is from a GDPR country.
    Returns True if in GDPR country or if check fails.
    """
    try:
        response = requests.get("https://ipinfo.io/json", timeout=2)
        if response.status_code == 200:
            data = response.json()
            # List of GDPR countries (EU member states + UK, Norway, Iceland, Liechtenstein, Switzerland)
            gdpr_countries = [
                "AT", "BE", "BG", "HR", "CY", "CZ", "DK", "EE", "FI", "FR",
                "DE", "GR", "HU", "IE", "IT", "LV", "LT", "LU", "MT", "NL",
                "PL", "PT", "RO", "SK", "SI", "ES", "SE", "GB", "UK", "NO",
                "IS", "LI", "CH"
            ]
            return data.get("country", "") in gdpr_countries
        return True  # If can't determine, assume GDPR to be safe
    except Exception:
        return True  # If check fails, assume GDPR to be safe


def analytics_disabled() -> bool:
    """
    Check if analytics should be disabled based on environment variables or GDPR status
    """
    env_disabled = os.getenv("GA_DISABLE_ANALYTICS")
    return bool(env_disabled) or is_gdpr_country()


class GoogleAnalyticsTracker:
    """Google Analytics event tracker for general usage"""

    @staticmethod
    def get_client_id():
        """Generate a UUID for client identification"""
        return str(uuid.uuid4())

    def __init__(
        self,
        measurement_id: str,
        api_secret: str,
        client_id: Optional[str] = None,
        package_params: Optional[Dict[str, str]] = None
    ):
        """
        Initialize Google Analytics tracker

        Args:
            measurement_id: GA4 measurement ID (format: G-XXXXXXXX)
            api_secret: GA4 API secret
            client_id: Optional client ID (will generate UUID if not provided)
            package_params: Optional default parameters to send with each event
        """
        self._disabled = analytics_disabled()

        # Configure API endpoint with debug mode if enabled
        base_url = (
            "https://www.google-analytics.com/mp/collect"
            if not _DEBUG else
            "https://www.google-analytics.com/debug/mp/collect"
        )
        self.base_url = f"{base_url}?measurement_id={measurement_id}&api_secret={api_secret}"

        self._client_id = client_id or self.get_client_id() if not self._disabled else None
        self._package = "netspresso"
        self._version = pkg_resources.get_distribution('netspresso').version
        self._package_params = package_params or {}

    def send_event(
        self,
        event_name: str,
        event_params: Optional[Dict[str, Any]] = None,
        user_id: Optional[str] = None,
        raise_errors: bool = False,
        _await_response: bool = False,  # Parameter kept for backward compatibility
    ) -> None:
        """
        Send an event to Google Analytics

        Args:
            event_name: Name of the event
            event_params: Optional parameters for the event
            user_id: Optional user ID for the event
            raise_errors: True to raise any errors that occur
            _await_response: Deprecated parameter (kept for backward compatibility)
        """
        if self._disabled:
            return

        if not event_params:
            event_params = {}

        params_copy = event_params.copy()  # Create a copy to avoid modifying the original

        # Add package info to parameters
        params_copy.update(self._package_params)
        params_copy["package"] = self._package
        params_copy["version"] = self._version

        # Create the payload
        payload = {
            "client_id": self._client_id,
            "events": [{"name": event_name, "params": params_copy}],
        }

        # Add user ID if provided
        if user_id:
            payload["user_id"] = user_id

        headers = {"Content-Type": "application/json"}
        data = json.dumps(payload)

        # Send request without logging
        with disable_request_logs():
            try:
                response = requests.post(self.base_url, headers=headers, data=data)
                response.raise_for_status()

                if _DEBUG:
                    print(f"GA Debug: {response.content}")
            except Exception as exception:
                if _DEBUG:
                    print(f"GA Error: {str(exception)}")

                if raise_errors:
                    raise exception


# Create reusable tracker based on environment variables (for easy integration)
def create_default_tracker() -> GoogleAnalyticsTracker:
    """Create a tracker using environment variables for configuration"""
    measurement_id = "G-L9L1VV8J87"  # os.getenv("GA_MEASUREMENT_ID")
    api_secret = "0xsQ54pBSy6M7r_tUi-xiA"  # os.getenv("GA_API_SECRET")

    return GoogleAnalyticsTracker(
        measurement_id=measurement_id,
        api_secret=api_secret,
    )


netspresso_analytics = create_default_tracker()



from pathlib import Path
from typing import List, Optional, Union

import qai_hub as hub
from qai_hub.client import Dataset, Device, Job, JobStatus, JobSummary, JobType, Model, SourceModel, SourceModelType

from netspresso.np_qai.options import Extension, Framework, Runtime


class NPQAIBase:
    def set_verbose(self, verbose: bool) -> None:
        hub.set_verbose(verbose)

    def upload_dataset(self, data, name=None) -> Dataset:
        """
        Upload a dataset to the QAI Hub.

        Args:
            data: The dataset to upload.
            name: The name of the dataset.

        Returns:
            Dataset: Returns a dataset object if successful.

        Note:
            For details, see `upload_dataset in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.upload_dataset.html>`_.
        """
        dataset = hub.upload_dataset(data=data, name=name)

        return dataset

    def get_dataset(self, dataset_id: str) -> Dataset:
        """
        Get a dataset from the QAI Hub.

        Args:
            dataset_id: The ID of the dataset to get.

        Returns:
            Dataset: Returns a dataset object if successful.

        Note:
            For details, see `get_dataset in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_dataset.html>`_.
        """
        dataset = hub.get_dataset(dataset_id=dataset_id)

        return dataset

    def get_datasets(self, offset: int = 0, limit: int = 50) -> List[Dataset]:
        """
        Get a list of datasets from the QAI Hub.

        Args:
            offset: The offset of the datasets to get even older datasets.
            limit: The limit of the datasets to get.

        Returns:
            List[Dataset]: Returns a list of dataset objects if successful.

        Note:
            For details, see `get_datasets in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_datasets.html>`_.
        """
        datasets = hub.get_datasets(offset=offset, limit=limit)

        return datasets

    def upload_model(self, model: Union[SourceModel, str], name: Optional[str] = None) -> Model:
        """
        Upload a model to the QAI Hub.

        Args:
            model: The model to upload.
            name: The name of the model.

        Returns:
            Model: Returns a model object if successful.

        Note:
            For details, see `upload_model in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.upload_model.html>`_.
        """
        model = hub.upload_model(model=model, name=name)

        return model

    def get_models(self, offset: int = 0, limit: int = 50) -> List[Model]:
        """
        Get a list of models from the QAI Hub.

        Args:
            offset: The offset of the models to get even older models.
            limit: The limit of the models to get.

        Returns:
            List[Model]: Returns a list of model objects if successful.

        Note:
            For details, see `get_models in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_models.html>`_.
        """
        models = hub.get_models(offset=offset, limit=limit)

        return models

    def get_model(self, model_id: str) -> Model:
        """
        Get a model from the QAI Hub.

        Args:
            model_id: The ID of the model to get.

        Returns:
            Model: Returns a model object if successful.

        Note:
            For details, see `get_model in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_model.html>`_.
        """
        model = hub.get_model(model_id=model_id)

        return model

    def get_devices(self, name: str = "", os: str = "", attributes: Union[str, List[str]] = None) -> List[Device]:
        """
        Get a list of devices from the QAI Hub.

        Args:
            name: The name of the device to get.
            os: The OS of the device to get.
            attributes: The attributes of the device to get.

        Returns:
            List[Device]: Returns a list of device objects if successful.

        Note:
            For details, see `get_devices in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_devices.html>`_.
        """
        if attributes is None:
            attributes = []
        devices = hub.get_devices(name=name, os=os, attributes=attributes)

        return devices

    def get_device_attributes(self) -> List[str]:
        """
        Get a list of device attributes from the QAI Hub.

        Returns:
            List[str]: Returns a list of device attribute strings if successful.

        Note:
            For details, see `get_device_attributes in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_device_attributes.html>`_.
        """
        device_attributes = hub.get_device_attributes()

        return device_attributes

    def get_job_summaries(
        self,
        offset: int = 0,
        limit: int = 50,
        creator: Optional[str] = None,
        state: Union[Optional[JobStatus.State], List[JobStatus.State]] = None,
        type: Optional[JobType] = None,
    ) -> List[JobSummary]:
        """
        Get a list of job summaries from the QAI Hub.

        Returns:
            List[JobSummary]: Returns a list of job summary objects if successful.

        Note:
            For details, see `get_job_summaries in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_job_summaries.html>`_.
        """
        job_summaries = hub.get_job_summaries(offset=offset, limit=limit, creator=creator, state=state, type=type)

        return job_summaries

    def get_job(self, job_id: str) -> Job:
        """
        Get a job from the QAI Hub.

        Args:
            job_id: The ID of the job to get.

        Returns:
            Job: Returns a job object if successful.

        Note:
            For details, see `get_job in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.get_job.html>`_.
        """
        job = hub.get_job(job_id=job_id)

        return job

    def get_source_extension(self, model_path):
        extension = Path(model_path).suffix

        return extension

    def get_framework(self, extension: Extension):
        if extension == Extension.ONNX:
            return Framework.ONNX
        elif extension == Extension.PT:
            return Framework.PYTORCH
        elif extension == Extension.AIMET:
            return Framework.AIMET
        elif extension == Extension.H5:
            return Framework.TENSORFLOW

    def get_target_extension(self, runtime=Runtime.TFLITE):
        runtime_extensions = {
            Runtime.TFLITE: ".tflite",
            Runtime.QNN_LIB_AARCH64_ANDROID: ".so",
            Runtime.QNN_CONTEXT_BINARY: ".bin",
            Runtime.ONNX: ".onnx",
            Runtime.PRECOMPILED_QNN_ONNX: ".zip",
        }

        return runtime_extensions.get(runtime)

    def get_display_runtime(self, runtime: Runtime) -> str:
        RUNTIME_DISPLAY_MAP = {
            Runtime.TFLITE: "TensorFlow Lite",
            Runtime.QNN_LIB_AARCH64_ANDROID: "Qualcomm® AI Engine Direct model library targeting AArch64 Android",
            Runtime.QNN_CONTEXT_BINARY: "Qualcomm® AI Engine Direct context binary targeting the hardware specified in the compile job.",
            Runtime.ONNX: "ONNX",
            Runtime.PRECOMPILED_QNN_ONNX: "ONNX Runtime model with a pre-compiled QNN context binary.",
        }
        return RUNTIME_DISPLAY_MAP.get(runtime, "Unknown runtime")

    def get_framework_by_runtime(self, runtime: Runtime):
        FRAMEWORK_RUNTIME_MAP = {
            Runtime.TFLITE: Framework.TFLITE,
            Runtime.QNN_LIB_AARCH64_ANDROID: Framework.QNN,
            Runtime.QNN_CONTEXT_BINARY: Framework.QNN,
            Runtime.ONNX: Framework.ONNX,
            Runtime.PRECOMPILED_QNN_ONNX: Framework.QNN,
        }
        return FRAMEWORK_RUNTIME_MAP.get(runtime, "Unknown framework")

    def get_framework_by_model_type(self, model_type: SourceModelType):
        FRAMEWORK_MODEL_TYPE_MAP = {
            SourceModelType.TORCHSCRIPT: Framework.PYTORCH,
            SourceModelType.TFLITE: Framework.TFLITE,
            SourceModelType.ONNX: Framework.ONNX,
            SourceModelType.MLMODEL: Framework.COREML,
            SourceModelType.MLMODELC: Framework.COREML,
            SourceModelType.MLPACKAGE: Framework.COREML,
            SourceModelType.TETRART: Framework.TENSORRT,
            SourceModelType.QNN_LIB_AARCH64_ANDROID: Framework.QNN,
            SourceModelType.QNN_LIB_X86_64_LINUX: Framework.QNN,
            SourceModelType.QNN_CONTEXT_BINARY: Framework.QNN,
            SourceModelType.AIMET_ONNX: Framework.AIMET,
        }
        return FRAMEWORK_MODEL_TYPE_MAP.get(model_type, "Unknown framework")

    def get_display_framework(self, framework: Framework):
        RUNTIME_DISPLAY_MAP = {
            Framework.PYTORCH: "PyTorch",
            Framework.ONNX: "ONNX",
            Framework.ONNXRUNTIME: "ONNXRuntime",
            Framework.AIMET: "AIMET",
            Framework.TENSORFLOW: "TensorFlow",
            Framework.TFLITE: "Tensorflow Lite",
            Framework.COREML: "CoreML",
            Framework.TENSORRT: "TensorRT",
            Framework.QNN: "QNN",
        }
        return RUNTIME_DISPLAY_MAP.get(framework, "Unknown runtime")
